"""
<name>Differentiation Scale</name>
<description></description>
<prototype>1</prototype>
"""

from __future__ import absolute_import

import os, sys
import random
from collections import defaultdict
from operator import itemgetter, add

import numpy

import Orange
from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets.OWWidget import *

from ... import obiDifscale

class OWDifferentiationScale(OWWidget):
    def __init__(self, parent=None, signalManager=None, title="Differentiation Scale"):
        OWWidget.__init__(self, parent, signalManager, title, wantGraph=True)
        
        self.inputs = [("Gene Expression Samples", Orange.data.Table, self.set_data), ("Additional Expression Samples", Orange.data.Table, self.set_additional_data)]
        self.outputs = [("Selected Time Points", Orange.data.Table), ("Additional Selected Time Points", Orange.data.Table)]
        
        self.selected_time_label = 0
        self.auto_commit = 0
        
        self.loadSettings()
        
        self.selection_changed_flag = False
        
        #####
        # GUI
        #####
        box = OWGUI.widgetBox(self.controlArea, "Info")
        self.info_label = OWGUI.widgetLabel(box, "No data on input")
        self.info_label.setWordWrap(True)
        self.info_label.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        
        OWGUI.rubber(self.controlArea)
        
        box = OWGUI.widgetBox(self.controlArea, "Selection")
        
        cb = OWGUI.checkBox(box, self, "auto_commit", "Commit on any change",
                            tooltip="Send updated selections automatically",
                            callback=self.commit_if)
        
        b = OWGUI.button(box, self, "Commit",
                         callback=self.commit,
                         tooltip="Send selections on output signals")
        
        OWGUI.setStopper(self, b, cb, "selection_changed_flag",
                         callback=self.commit)
        
        self.connect(self.graphButton, SIGNAL("pressed()"), self.save_graph)
        
        self.scene = QGraphicsScene()
        self.scene_view = DiffScaleView(self.scene, self.mainArea)
        self.scene_view.setRenderHint(QPainter.Antialiasing)
        self.scene_view.setMinimumWidth(300)
        self.mainArea.layout().addWidget(self.scene_view)
        self.connect(self.scene, SIGNAL("selectionChanged()"), self.on_selection_changed)
        self.connect(self.scene_view, SIGNAL("view_resized(QSize)"), lambda size: self.on_view_resized())
        
        self.data = None
        self.additional_data = None
        self.projections1 = []
        self.projections2 = []
        self.labels1 = []
        self.labels2 = []
        
        self.selected_time_samples = [], []
        
        self.controlArea.setMaximumWidth(300)
        self.resize(600, 480)
        
    def clear(self):
        """ Clear the widget state
        """
        self.projections1 = []
        self.projections2 = []
        self.labels1 = []
        self.labels2 = []
        self.clear_selection()
        self.scene.clear()
        
    def clear_selection(self):
        """ Clear the time point selection.
        """
        self.selected_time_samples = [], []
        
    def set_data(self, data = None):
        """ Set the data for the widget.
        """
        self.clear()
        self.data = data
        
    def set_additional_data(self, data=None):
        """ Set an additional data set.
        """
        self.clear()
        self.additional_data = data
        
    def handleNewSignals(self):
        if self.data is not None:
            self.run_projections()
            self.projection_layout()
            self.update_graph()
            
            info_text = """\
Data with {0} genes 
and {1} samples on input.\n""".format(len(self.data),
                 len(self.data.domain.attributes))
            if self.additional_data is not None:
                info_text += """\
Additional data with {0} genes
and  {1} samples on input.""".format(len(self.additional_data),
                                                    len(self.additional_data.domain.attributes))
            self.info_label.setText(info_text)
        else:
            self.send("Selected Time Points", None)
            self.send("Additional Selected Time Points", None)
            self.info_label.setText("No data on input\n")
            
    def run_projections(self):
        """ Run obiDifscale.get_projections with the current inputs.
        """
        self.error()
#        try:
#            attr_set = list(set(a.attributes['time'] for a in data.domain.attributes))
#            self.time_points = obiDifscale.conv(attr_set, ticks=False)
#        except KeyError, ex:
#            self.error("Could not extract time data")
#            self.clear()
#            return
        
        try:
            (self.projections1, self.labels1,
             self.projections2, self.labels2) = \
                obiDifscale.get_projections(self.data, data2=self.additional_data)
        except Exception, ex:
            self.error("Failed to obtain the projections due to: %r" % ex)
            self.clear()
            return
        
    def projection_layout(self):
        """ Compute the layout for the projections.
        """
        if self.projections1: 
            projections = self.projections1 + self.projections2
            projections = numpy.array(projections)
            
            x_min = numpy.min(projections)
            x_max = numpy.max(projections)
            
            # Scale projections 
            projections = (projections - x_min) / ((x_max - x_min) or 1.0)
            projections = list(projections)
            
            labels = self.labels1 + self.labels2
            
            samples = [(attr, self.data) for attr in self.data.domain.attributes] + \
                      ([(attr, self.additional_data) for attr in self.additional_data.domain.attributes] \
                       if self.additional_data is not None else [])
            
            # TODO: handle samples with the same projection
            # the point_layout should return the proj to sample mapping instead
            proj_to_sample = dict([((label, proj), sample) for label, proj, sample \
                                   in zip(labels, projections, samples)])
            self.proj_to_sample = proj_to_sample
            
            time_points = point_layout(labels, projections)
            self.time_points = time_points
            level_height = 20
            all_points = numpy.array(reduce(add, [p for _, p in time_points], []))
            self.all_points = all_points
            
#            all_points[:, 1] *= -level_height
            self.time_samples = [] # samples for time label (same order as in self.time_points)
            
            point_i = 0
            for label, points, in time_points:
                samples = [] 
                for x, y in points:
                    samples.append(proj_to_sample.get((label, x), None))
                self.time_samples.append((label, samples))
            
    def update_graph(self):
        """ Populate the Graphics Scene with the current projections. 
        """
        scene_size_hint = self.scene_view.viewport().size()
        scene_size_hint = QSizeF(max(scene_size_hint.width() - 50, 100),
                                 scene_size_hint.height())
        self.scene.clear()
        
        if self.projections1:
            level_height = 20
            all_points = self.all_points.copy()
            all_points[:, 0] *= scene_size_hint.width()
            all_points[:, 1] *= -level_height
            
            point_i = 0
            centers = []
            z_value = 0
            for label, samples in self.time_samples:
                # Points
                p1 = all_points[point_i]
                points = all_points[point_i: point_i + len(samples), :]
                for (x, y), sample in zip(points, samples):
                    item = GraphicsTimePoint(QRectF(QPointF(x-3, y-3), QSizeF(6, 6)))
                    item.setBrush(QBrush(Qt.black))
                    item.sample = sample
                    item.setToolTip(sample[0].name if sample else "")
                    item.setZValue(z_value)
                    self.scene.addItem(item)
                    point_i += 1
                p2 = all_points[point_i - 1]
                
                # Line over all points
                line = QGraphicsLineItem(QLineF(*(tuple(p1) + tuple(p2))))
                line.setPen(QPen(Qt.black, 2))
                line.setZValue(z_value - 1)
                self.scene.addItem(line)
                
                # Time label on top of the median
                n_points = len(points)
                if n_points % 2:
                    center = points[n_points / 2]
                else:
                    center = (points[n_points / 2] + points[n_points / 2 + 1]) / 2.0
                centers.append(center)
                x, y = center
                text = QGraphicsSimpleTextItem(label)
                w = text.boundingRect().width()
                text.setPos(x - w / 2.0, y - 17.5)
                self.scene.addItem(text)
            
            self.scene.addLine(QLineF(0.0, 0.0, scene_size_hint.width(), 0.0))
            
            polygon = QPolygonF([QPointF(3.0, 0.0),
                                 QPointF(-2.0, -2.0),
                                 QPointF(0.0, 0.0),
                                 QPointF(-2.0, 2.0),
                                 QPointF(3.0, 0.0)])
            
            arrow = QGraphicsPolygonItem(polygon)
            arrow.setBrush(QBrush(Qt.black))
            arrow.setPos(scene_size_hint.width(), 0.0)
            arrow.scale(2, 2)
            self.scene.addItem(arrow)
            
            title = QGraphicsSimpleTextItem("Development (time)")
            font = self.font()
            font.setPointSize(10)
            title.setFont(font)
            w = title.boundingRect().width()
            title.setPos(scene_size_hint.width() - w, -15)
            self.scene.addItem(title)
            
            rects = []
            ticks = []
            axis_label_items = []
            labels = [(center, label) for center, (label, _) in zip(centers, self.time_samples)]
            labels = sorted(labels, key=lambda (c, l): c[0])
            for center, label in labels:
                x, y = center
                item = QGraphicsSimpleTextItem(label)
                w = item.boundingRect().width()
                item.setPos(x - w / 2.0, 4.0)
                rects.append(item.sceneBoundingRect().normalized())
                ticks.append(QPointF(x - w / 2.0, 4.0))
                axis_label_items.append(item)
            
#            rects = SA_axis_label_layout(ticks, rects, max_time=0.5,
#                                         x_factor=scene_size_hint.width() / 50.0,
#                                         y_factor=10,
#                                         random=random.Random(0))

            rects = greedy_scale_label_layout(ticks, rects, spacing=5)
            
            for (tick, label), rect, item in zip(labels, rects, axis_label_items):
                x, y = tick
                self.scene.addLine(x, -2, x, 2)
                if rect.top() - item.pos().y() > 5:
                    self.scene.addLine(x, 2, rect.center().x(), 14.0)
                if rect.top() - item.pos().y() > 15:
                    self.scene.addLine(rect.center().x(), 14.0, rect.center().x(), rect.top())
#                item.setPos(rect.topLeft())
                
#                text = QGraphicsSimpleTextItem(label)
#            for tick, rect, item in zip(ticks, rects, axis_label_items):
                item.setPos(rect.topLeft())
                self.scene.addItem(item)
#                w = text.boundingRect().width()
#                text.setPos(x - w / 2.0, 4)
                # Need to compute axis label layout.
#                self.scene.addItem(text)

            self.scene.setSceneRect(self.scene.itemsBoundingRect().adjusted(-10, -10, 10, 10))

    def on_view_resized(self):
        self.update_graph()
        
    def on_selection_changed(self):
        try:
            selected = self.scene.selectedItems()
        except RuntimeError:
            return
        
        selected_attrs1 = []
        selected_attrs2  =[]
        for point in selected:
            attr, data = point.sample if point.sample else (None, None)
            if data is self.data:
                selected_attrs1.append(attr)
            elif data is self.additional_data:
                selected_attrs2.append(attr)
                
        self.selected_time_samples = selected_attrs1, selected_attrs2
        print self.selected_time_samples
        self.commit_if()
            
    def commit_if(self):
        if self.auto_commit:
            self.commit()
        else:
            self.selection_changed_flag = True
    
    def commit(self):
        if self.data is not None:
            selected1, selected2 = self.selected_time_samples
            attrs1 = [a for a in self.data.domain.attributes \
                      if a in selected1]
            domain = Orange.data.Domain(attrs1, self.data.domain.class_var)
            domain.add_metas(self.data.domain.get_metas())
            data = Orange.data.Table(domain, self.data)
            self.send("Selected Time Points", data)
            
            if self.additional_data is not None:
                attrs2 = [a for a in self.additional_data.domain.attributes \
                          if a in selected2]
                domain = Orange.data.Domain(attrs2, self.additional_data.domain.class_var)
                domain.add_metas(self.additional_data.domain.get_metas())
                data = Orange.data.Table(domain, self.additional_data)
                self.send("Additional Selected Time Points", data)
        else:
            self.send("Selected Time Points", None)
            self.send("Additional Selected Time Points", None)
        self.selection_changed_flag = False
        
    def save_graph(self):
        from Orange.OrangeWidgets.OWDlgs import OWChooseImageSizeDlg
        dlg = OWChooseImageSizeDlg(self.scene, parent=self)
        dlg.exec_()
    
    
class GraphicsTimePoint(QGraphicsEllipseItem):
    def __init__(self, *args):
        QGraphicsEllipseItem.__init__(self, *args)
        self.setFlags(QGraphicsItem.ItemIsSelectable)
        self.setAcceptsHoverEvents(True)
        self._is_hovering = False
        
    def paint(self, painter, option, widget=0):
        if self.isSelected():
            brush = QBrush(Qt.red)
            pen = QPen(Qt.red, 1)
        else:
            brush = QBrush(Qt.darkGray)
            pen = QPen(Qt.black, 1)
        if self._is_hovering:
            brush = QBrush(brush.color().darker(200))
        painter.save()
        painter.setBrush(brush)
        painter.setPen(pen)
        painter.drawEllipse(self.rect())
        painter.restore()
        
    def hoverEnterEvent(self, event):
        self._is_hovering = True
        self.update()
        return QGraphicsEllipseItem.hoverEnterEvent(self, event)
    
    def hoverLeaveEvent(self, event):
        self._is_hovering = False
        self.update()
        return QGraphicsEllipseItem.hoverLeaveEvent(self, event)
        
    
class DiffScaleView(QGraphicsView):
    def resizeEvent(self, event):
        QGraphicsView.resizeEvent(self, event)
        self.emit(SIGNAL("view_resized(QSize)"), event.size())
        

def point_layout(labels, points, label_size_hints=None):
    groups = defaultdict(list)
    for label, point in zip(labels, points):
        groups[label].append(point)
        
    for label, points in list(groups.items()):
        points = sorted(points)
        # TODO: Use label_size_hints for min, max
        groups[label] = (points, (points[0], points[-1]))
    
    sorted_groups = sorted(groups.items(), key=itemgetter(1), reverse=True)
    levels = {}
    curr_level = 1
    label_levels = {}
    while sorted_groups:
        label, (points, (x_min, x_max)) = sorted_groups.pop(-1)
        max_level_pos = levels.get(curr_level, x_min)
        if x_min < max_level_pos:
            curr_level += 1
            sorted_groups.append((label, (points, (x_min, x_max))))
        else:
            label_levels[label] = curr_level
            levels[curr_level] = x_max
            curr_level = 1
            
    for label, (points, _) in list(groups.items()):
        level = float(label_levels[label])
        groups[label] = [(x, level) for x in points]
        
    return list(groups.items())

    
def greedy_scale_label_layout(ticks, rects, spacing=3):
    """ Layout the labels at ticks on a linear scale, by raising the
    overlapping labels.
    
    """
    def adjust_interval(start, end, min_v, max_v):
        """ Adjust (start, end) interval to fit inside the (min_v, max_v).
        """
        if start < min_v:
            return (min_v, min_v + (end - start))
        elif max_v > end:
            return (max_v - (end - start), max_v)
        else:
            return (start, end)
        
    def center_interval(start, end, center):
        """ Center the interval on `center`
        """
        span = end - start
        return centered(center, span)
    
    def centered(center, span):
        """ Return an centered interval with span.
        """
        return (center - span / 2.0, center + span / 2.0)
    
    def contains((start, end), (start1, end1)):
        return start <= start1  and end >= end1
    
    def fit(work, ticks, min_x, max_x):
        """ Fit the work set between min_x and max_x  and centered on the
        ticks, if possible.
        """
        fits = False
        work_set = map(QRectF, work)
        tick_center = sum([r.center().x() for r in work_set]) / len(work_set)
        if len(work_set) == 1:
            if work_set[0].left() >= min_x and work_set[0].right() <= max_x:
                return work_set
            else:
                return []
        
        elif len(work_set) == 2: # TODO: MErge this with the > 2
            w_sum = sum([r.width() for r in work_set]) + spacing
            if w_sum < max_x - min_x:
                r1, r2 = work_set
                interval = centered(tick_center, w_sum)
                
                if not contains((min_x, max_x), interval):
                    interval = adjust_interval(*(interval + (min_x, max_x)))
                    
                if contains((min_x, max_x), interval):
                    r1.moveLeft(interval[0])
                    r2.moveLeft(interval[1] - r2.width())
                    r1.moveTop(r1.top() + 10)
                    r2.moveTop(r2.top() + 10)
                    return work_set
                else:
                    return []
            else:
                return []
        
        elif len(work_set) > 2:
            center = (work_set[0].center().x() + work_set[-1].center().x()) / 2.0
            w_sum = work_set[0].width() / 2.0 + work_set[-1].width() / 2.0 + spacing
            for i, r in enumerate(work_set[1:-1]):
                w_sum += r.width() + spacing
            interval = centered(center, w_sum)
            
            if not contains((min_x, max_x), interval):
                interval = adjust_interval(*(interval + (min_x, max_x)))
                
            if contains((min_x, max_x), interval):
                istart, iend = interval
                rstart, rend = work_set[0], work_set[-1]
                rstart.moveLeft(istart)
                rstart.moveTop(rstart.top() + 10)
                rend.moveLeft(iend - rend.width())
                rend.moveTop(rend.top() + 10)
                istart += rstart.width() / 2.0
                iend -= rend.width() / 2.0
                for r in work_set[1: -1]:
                    r.moveLeft(istart)
                    r.moveTop(r.top() + 20)
                    istart += r.width() + spacing
                return work_set
            else:
                return []
            
    queue = sorted(zip(ticks, rects),
                   key=lambda (t, _): t.x(),
                   reverse=True)
    done = False
    rects = []
    
    min_x = -1e30
    max_x = 1e30
    
    while queue:
        work_set = [queue.pop(-1)]
        set_fits = False
        max_x = queue[-1][1].left() if queue else 1e30
        while not set_fits:
            new_rects = fit(map(itemgetter(1), work_set),
                            map(itemgetter(0), work_set),
                            min_x, max_x)
            if new_rects: # Can the work set be fit.
                set_fits = True
                rects.extend(new_rects)
                min_x = work_set[-1][1].right()
                
            else:
                # Extend the work set with one more label rect
                work_set.append(queue.pop(-1))
                max_x = queue[-1][1].left() if queue else 1e30
    return rects
        
    
if __name__ == "__main__":
    app = QApplication(sys.argv)
    w = OWDifferentiationScale()
    data = Orange.data.Table(os.path.expanduser("~/Documents/GDS2666n"))
    w.show()
    w.set_data(data)
    w.handleNewSignals()
    app.exec_()
    w.saveSettings()



"""
Construction of gene set scores for each sample.

Learners in this module build models needed for construction
of features from individual genes.

The other classes just take example and return a
dictionary of { name: score } for that example.
"""

from __future__ import absolute_import

from collections import defaultdict
import math

import numpy

import orange, Orange, statc

from . import obiExpression, obiGene, obiGsea, obiGeneSets, stats

def normcdf(x, mi, st):
    return 0.5*(2. - stats.erfcc((x - mi)/(st*math.sqrt(2))))

class AT_edelmanParametric(object):

    def __init__(self, **kwargs):
        for a,b in kwargs.items():
            setattr(self, a, b)

    def __call__(self, nval):

        if self.mi1 == None or self.mi2 == None or self.st1 == None or self.st2 == None:
            return 0 

        try:
            val = nval.value
            if nval.isSpecial():
                return 0 
        except:
            val = nval

        try:
            if val >= self.mi1:
                p1 = 1 - normcdf(val, self.mi1, self.st1)
            else:
                p1 = normcdf(val, self.mi1, self.st1)

            if val >= self.mi2:
                p2 = 1 - normcdf(val, self.mi2, self.st2)
            else:
                p2 = normcdf(val, self.mi2, self.st2)

            #print p1, p2
            return math.log(p1/p2)
        except:
            #print p1, p2, "exception"
            return 0

class AT_edelmanParametricLearner(object):
    """
    Returns attribute transfromer for Edelman parametric measure for a given attribute in the
    dataset.
    Edelman et al, 06.

    Modified a bit.
    """

    def __init__(self, a=None, b=None):
        """
        a and b are choosen class values.
        """
        self.a = a
        self.b = b

    def __call__(self, i, data):
        cv = data.domain.classVar
        #print data.domain

        if self.a == None: self.a = cv.values[0]
        if self.b == None: self.b = cv.values[1]

        def avWCVal(value):
            return [ex[i].value for ex in data if ex[-1].value == value and not ex[i].isSpecial() ]

        list1 = avWCVal(self.a)
        list2 = avWCVal(self.b)

        mi1 = mi2 = st1 = st2 = None

        try:
            mi1 = statc.mean(list1)
            st1 = statc.std(list1)
        except:
            pass
    
        try:
            mi2 = statc.mean(list2)
            st2 = statc.std(list2)
        except:
            pass

        return AT_edelmanParametric(mi1=mi1, mi2=mi2, st1=st1, st2=st2)

class AT_loess(object):

    def __init__(self, **kwargs):
        for a,b in kwargs.items():
            setattr(self, a, b)

    def __call__(self, nval):

        val = nval.value
        if nval.isSpecial():
            return 0.0 #middle value
        #return first class probablity

        def saveplog(a,b):
            try:
                return math.log(a/b)
            except:
                if a < b:
                    return -10
                else:
                    return +10

        try:
            ocene = self.condprob(val)
            if sum(ocene) < 0.01:
                return 0.0
            return saveplog(ocene[0], ocene[1])

        except:
            return 0.0

class AT_loessLearner(object):

    def __call__(self, i, data):
        cv = data.domain.classVar
        #print data.domain
        try:
            ca = orange.ContingencyAttrClass(data.domain.attributes[i], data)
            a = orange.ConditionalProbabilityEstimatorConstructor_loess(ca, nPoints=5)
            return AT_loess(condprob=a)
        except:
            return AT_loess(condprob=None)

def nth(l, n):
    return [a[n] for a in l]

class Assess(object):

    def __init__(self, **kwargs):
        for a,b in kwargs.items():
            setattr(self, a, b)

    def __call__(self, example):
        enrichmentScores = [] 

        lcor = [ self.attrans[at](example[at]) for at in range(len(self.attrans)) ]

        ordered = obiGsea.orderedPointersCorr(lcor)

        def rev(l):
           return numpy.argsort(l)

        rev2 = rev(ordered)

        gsetsnumit = self.gsetsnum.items()

        enrichmentScores = dict( 
            (name, obiGsea.enrichmentScoreRanked(subset, lcor, ordered, rev2=rev2)[0]) \
            for name,subset in gsetsnumit)
    
        return enrichmentScores


class AssessLearner(object):
    """
    Uses the underlying GSEA code to select genes.
    Takes data and creates attribute transformations.
    """
    
    def __call__(self, data, matcher, geneSets, minSize=3, maxSize=1000, minPart=0.1, classValues=None, rankingf=None):
        data, oknames, gsetsnum = selectGenesetsData(data, matcher, geneSets, \
            minSize=minSize, maxSize=maxSize, minPart=minPart, classValues=classValues)
        
        if rankingf == None:
            rankingf = AT_edelmanParametricLearner()

        #rank individual attributes on the training set
        attrans = [ rankingf(iat, data) for iat, at in enumerate(data.domain.attributes) ]

        return Assess(attrans=attrans, gsetsnum=gsetsnum)

def selectGenesetsData(data, matcher, geneSets, minSize=3, maxSize=1000, minPart=0.1, classValues=None):
    """
    Returns gene sets and data which falling under upper criteria.
    """
    gso = obiGsea.GSEA(data, matcher=matcher, classValues=classValues, atLeast=0)
    gso.addGenesets(geneSets)
    okgenesets = gso.selectGenesets(minSize=minSize, maxSize=maxSize, minPart=minPart).keys()
    gsetsnum = gso.to_gsetsnum(okgenesets)
    return gso.data, okgenesets, gsetsnum

def corgs_activity_score(ex, corg):
    """ activity score for a sample for pathway given by corgs """
    #print [ ex[i].value for i in corg ] #FIXME what to do with unknown values?
    return sum(ex[i].value if ex[i].value != '?' else 0.0 for i in corg)/len(corg)**0.5

class CORGs(object):

    def __init__(self, **kwargs):
        for a,b in kwargs.items():
            setattr(self, a, b)

    def __call__(self, example):
        return dict( (name,corgs_activity_score(example, corg)) \
            for name, corg in self.corgs.items() )

class CORGsLearner(object):
    
    def __call__(self, data, matcher, geneSets, minSize=3, maxSize=1000, minPart=0.1, classValues=None):
        """
        WARNING: input has to be z_ij table! each gene needs to be normalized
        (mean=0, stdev=1) for all samples.
        """
        data, oknames, gsetsnum = selectGenesetsData(data, matcher, geneSets, \
            minSize=minSize, maxSize=maxSize, minPart=minPart, classValues=classValues)
    
        tscorecache = {}

        def tscorec(data, at, cache=None):
            """ Cached attribute  tscore calculation """
            if cache != None and at in cache: return cache[at]
            ma = obiExpression.MA_t_test()(at,data)
            if cache != None: cache[at] = ma
            return ma

        def compute_corg(data, inds):
            """
            Compute CORG for this geneset specified with gene inds
            in the example table. Output is the list of gene inds
            in CORG.

            """
            #order member genes by their t-scores: decreasing, if av(t-score) >= 0,
            #else increasing
            tscores = [ tscorec(data, at, tscorecache) for at in inds ]
            sortedinds = nth(sorted(zip(inds,tscores), key=lambda x: x[1], \
                reverse=statc.mean(tscores) >= 0), 0)

            def S(corg):
                """ Activity score separation - S(G) in 
                the article """
                asv = orange.FloatVariable(name='AS')
                asv.getValueFrom = lambda ex,rw: orange.Value(asv, corgs_activity_score(ex, corg))
                data2 = orange.ExampleTable(orange.Domain([asv], data.domain.classVar), data)
                return abs(tscorec(data2, 0)) #FIXME absolute - nothing in the article about it
                    
            #greedily find CORGS procing the best separation
            g = S(sortedinds[:1])
            bg = 1
            for a in range(2, len(sortedinds)+1):
                tg = S(sortedinds[:a])
                if tg > g:
                    g = tg
                    bg = a
                else:
                    break
                
            return sortedinds[:a]

        #now, on the learning set produce the CORGS for each geneset and
        #save it for use in further prediction

        corgs = {}

        for name, inds in gsetsnum.items():
            inds = sorted(set(inds)) # take each gene only once!
            corgs[name] = compute_corg(data, inds)

        return CORGs(corgs=corgs)

class GSA(object):

    def __init__(self, **kwargs):
        for a,b in kwargs.items():
            setattr(self, a, b)

    def __call__(self, example):
        return dict( (name, statc.mean([example[i].value for i in inds if example[i].value != "?"]) ) \
            for name, inds in self.subsets.items() )

class GSALearner(object):
    
    def __call__(self, data, matcher, geneSets, minSize=3, maxSize=1000, minPart=0.1, classValues=None):
        """
        WARNING: input has to be z_ij table! each gene needs to be normalized
        (mean=0, stdev=1) for all samples.
        """
        import scipy.stats

        data, oknames, gsetsnum = selectGenesetsData(data, matcher, geneSets, \
            minSize=minSize, maxSize=maxSize, minPart=minPart, classValues=classValues)
    
        def tscorec(data, at, cache=None):
            ma = obiExpression.MA_t_test()(at,data)
            return ma

        tscores = [ tscorec(data, at) for at in data.domain.attributes ]

        def to_z_score(t):
            return float(scipy.stats.norm.ppf(scipy.stats.t.cdf(t, len(data)-2)))

        zscores = map(to_z_score, tscores)

        subsets = {}

        for name, inds in gsetsnum.items():
            inds = sorted(set(inds)) # take each gene only once!

            D = statc.mean([max(zscores[i],0) for i in inds]) \
                + statc.mean([min(zscores[i],0) for i in inds])

            if D >= 0:
                subsets[name] = [ i for i in inds if zscores[i] > 0.0 ]
            else:
                subsets[name] = [ i for i in inds if zscores[i] < 0.0 ]

        return GSA(subsets=subsets)

def pls_transform(example, constt):
    """
    Uses calculated PLS weights to transform the given example.
    """

    inds, xmean, W, P = constt
    dom = orange.Domain([example.domain.attributes[i1] for i1 in inds ], False)
    newex = orange.ExampleTable(dom, [example])
    ex = newex.toNumpy()[0]
    ex = ex - xmean # same input transformation

    nc = W.shape[1]

    TR = numpy.empty((1, nc))
    XR = ex

    dot = numpy.dot

    for i in range(nc):
       t = dot(XR, W[:,i].T)
       XR = XR - t*numpy.array([P[:,i]])
       TR[:,i] = t

    return TR

def PLSCall(data, y=None, x=None, nc=None, weight=None, save_partial=False):

    def normalize(vector):
        return vector / numpy.linalg.norm(vector)

    if y == None:
        y = [ data.domain.classVar ]
    if x == None:
        x = [v for v in data.domain.variables if v not in y]

    Ncomp = nc if nc is not None else len(x)
        
    dataX = orange.ExampleTable(orange.Domain(x, False), data)
    dataY = orange.ExampleTable(orange.Domain(y, False), data)

    # transformation to numpy arrays
    X = dataX.toNumpy()[0]
    Y = dataY.toNumpy()[0]

    # data dimensions
    n, mx = numpy.shape(X)
    my = numpy.shape(Y)[1]

    # Z-scores of original matrices
    YMean = numpy.mean(Y, axis = 0)
    XMean = numpy.mean(X, axis = 0)
    
    X = (X-XMean)
    Y = (Y-YMean)

    P = numpy.empty((mx,Ncomp))
    T = numpy.empty((n,Ncomp))
    W = numpy.empty((mx,Ncomp))
    E,F = X,Y

    dot = numpy.dot
    norm = numpy.linalg.norm

    #PLS1 - from Gutkin, shamir, Dror: SlimPLS

    for i in range(Ncomp):
        w = dot(E.T,F)
        w = w/norm(w) #normalize w in Gutkin et al the do w*c, where c is 1/norm(w)
        t = dot(E, w) #t_i -> a row vector
        p = dot(E.T, t)/dot(t.T, t) #p_i t.T is a row vector - this is inner(t.T, t.T)
        q = dot(F.T, t)/dot(t.T, t) #q_i
            
        E = E - dot(t, p.T)
        F = F - dot(t, q.T)

        T[:,i] = t.T
        W[:,i] = w.T
        P[:,i] = p.T

    return XMean, W, P, T

class PLS(object):

    def __init__(self, **kwargs):
        for a,b in kwargs.items():
            setattr(self, a, b)

    def __call__(self, example):

        od = {}

        for name, constt in self.constructt.items():
            ts = pls_transform(example, constt)[0]
            if len(ts) == 1:
                od[name] = ts[0]
            else:
                for i,s in enumerate(ts):
                   od[name + "_LC_" + str(i+1)] = s
 
        return od

class PLSLearner(object):
    """ Transforms gene sets using Principal Leasts Squares. """
    
    def __call__(self, data, matcher, geneSets, minSize=3, maxSize=1000, minPart=0.1, classValues=None, components=1):
        """
        If more that 1 components are used, _LC_componetsNumber is appended to 
        the name of the gene set.
        """

        data, oknames, gsetsnum = selectGenesetsData(data, matcher, geneSets, \
            minSize=minSize, maxSize=maxSize, minPart=minPart, classValues=classValues)
    
        constructt = {}

        #build weight matrices for every gene set
        for name, inds in gsetsnum.items():
            dom2 = orange.Domain([ data.domain.attributes[i1] for i1 in inds ], data.domain.classVar)
            data_gs = orange.ExampleTable(dom2, data)
            xmean, W, P, T = PLSCall(data_gs, nc=components, y=[data_gs.domain.classVar], save_partial=True)
            constructt[name] = inds, xmean, W, P

        return PLS(constructt=constructt)

class PCA(object):

    def __init__(self, **kwargs):
        for a,b in kwargs.items():
            setattr(self, a, b)

    def __call__(self, example):
        od = {}

        for name, constt in self.constructt.items():
            ts = pca_transform(example, constt)[0]
            od[name] = ts

        return od

def pca_transform(example, constt):
    inds, evals, evect, xmean = constt
    dom = orange.Domain([example.domain.attributes[i1] for i1 in inds ], False)
    newex = orange.ExampleTable(dom, [example])
    arr = newex.toNumpy()[0]
    arr = arr - xmean # same input transformation - a row in a matrix

    ev0 = evect[0] #this is a row in a matrix - do a dot product
    a = numpy.dot(arr, ev0)
    return a

def pca(data, snapshot=0):
    "Perform PCA on M, return eigenvectors and eigenvalues, sorted."
    M = data.toNumpy("a")[0]
    XMean = numpy.mean(M, axis = 0)
    M = M - XMean

    T, N = numpy.shape(M)
    # if there are less rows T than columns N, use snapshot method
    if (T < N) or snapshot:
        C = numpy.dot(M, numpy.transpose(M))
        evals, evecsC = numpy.linalg.eigh(C) #columns of evecsC are eigenvectors
        evecs = numpy.dot(M.T, evecsC)/numpy.sqrt(numpy.abs(evals))
    else:
        K = numpy.dot(numpy.transpose(M), M)
        evals, evecs = numpy.linalg.eigh(K)
    
    evecs = numpy.transpose(evecs)

    # sort the eigenvalues and eigenvectors, decending order
    order = (numpy.argsort(numpy.abs(evals))[::-1])
    evecs = numpy.take(evecs, order, 0)
    evals = numpy.take(evals, order)
    return evals, evecs, XMean

class PCALearner(object):
    """ Transforms gene sets using Principal Leasts Squares. """
    
    def __call__(self, data, matcher, geneSets, minSize=3, maxSize=1000, minPart=0.1, classValues=None):

        data, oknames, gsetsnum = selectGenesetsData(data, matcher, geneSets, \
            minSize=minSize, maxSize=maxSize, minPart=minPart, classValues=classValues)
    
        constructt = {}

        #build weight matrices for every gene set
        for name, inds in gsetsnum.items():
            dom2 = orange.Domain([ data.domain.attributes[i1] for i1 in inds ], data.domain.classVar)

            data_gs = orange.ExampleTable(dom2, data)
            evals, evect, xmean = pca(data_gs)
            constructt[name] = inds, evals, evect, xmean

        return PCA(constructt=constructt)


class SimpleFun(object):

    def __init__(self, **kwargs):
        for a,b in kwargs.items():
            setattr(self, a, b)

    def __call__(self, example):
        #for  name,ids in self.gsets.items():
        #    print name, [example[i].value for i in ids], self.fn([example[i].value for i in ids])
        return dict( (name, self.fn([example[i].value for i in ids])) \
            for name,ids in self.gsets.items() )

class SimpleFunLearner(object):
    """
    Just applies a function taking attribute values of an example and
    produces to all gene sets.    
    """
    def __call__(self, data, matcher, geneSets, minSize=3, maxSize=1000, minPart=0.1, classValues=None, fn=None):
        data, oknames, gsetsnum = selectGenesetsData(data, matcher, geneSets, \
            minSize=minSize, maxSize=maxSize, minPart=minPart, classValues=classValues)
        return SimpleFun(gsets=gsetsnum, fn=fn)

class MedianLearner(object):
    
    def __call__(self, data, matcher, geneSets, minSize=3, maxSize=1000, minPart=0.1, classValues=None, fn=None):
       sfl =  SimpleFunLearner()
       return sfl(data, matcher, geneSets, \
            minSize=minSize, maxSize=maxSize, minPart=minPart, classValues=classValues, fn=statc.median)

class MeanLearner(object):
    
    def __call__(self, data, matcher, geneSets, minSize=3, maxSize=1000, minPart=0.1, classValues=None, fn=None):
       sfl =  SimpleFunLearner()
       return sfl(data, matcher, geneSets, \
            minSize=minSize, maxSize=maxSize, minPart=minPart, classValues=classValues, fn=statc.mean)

def impute_missing(data):
    #remove attributes with only unknown values
    newatts = []
    for at in data.domain.attributes:
        svalues = [ 1 for a in data if a[at].isSpecial() ]
        real = len(data) - len(svalues)
        if real > 0:
            newatts.append(at)

    dom2 = orange.Domain(newatts, data.domain.classVar)
    data = orange.ExampleTable(dom2, data)

    #impute
    from Orange.orng import orngTree 
    imputer = orange.ImputerConstructor_model() 
    imputer.learnerContinuous = imputer.learnerDiscrete = orange.MajorityLearner()
    imputer = imputer(data)

    data = imputer(data)
    return data

def setSig_example(ldata, ex, genesets):
    """
    Create an dictionary with (geneset_name, geneset_expression) for every
    geneset on input.

    ldata is class-annotated data
    """
    #seznam ocen genesetov za posamezen primer v ucni mzozici
    pathways = {}

    def setSig_example_geneset(ex, data):
        """ ex contains only selected genes """

        distances = [ [], [] ]    

        def pearsonr(v1, v2):
            try:
                return statc.pearsonr(v1, v2)[0]
            except:
                return numpy.corrcoef([v1, v2])[0,1]

        def pearson(ex1, ex2):
            attrs = range(len(ex1.domain.attributes))
            vals1 = [ ex1[i].value for i in attrs ]
            vals2 = [ ex2[i].value for i in attrs ]
            return pearsonr(vals1, vals2)

        def ttest(ex1, ex2):
            try:
                return stats.lttest_ind(ex1, ex2)[0]
            except:
                return 0.0
        
        #maps class value to its index
        classValueMap = dict( [ (val,i) for i,val in enumerate(data.domain.classVar.values) ])
     
        #create distances to all learning data - save or other class
        for c in data:
            distances[classValueMap[c[-1].value]].append(pearson(c, ex))

        return ttest(distances[0], distances[1])
           
    for name, gs in genesets.items(): #for each geneset
        #for each gene set: take the attribute subset and work on the attribute subset only
        #only select the subset of genes from the learning data
        domain = orange.Domain([ldata.domain.attributes[ai] for ai in gs], ldata.domain.classVar)
        datao = orange.ExampleTable(domain, ldata)
        example = orange.Example(domain, ex) #domains need to be the same
      
        ocena = setSig_example_geneset(example, datao)
        pathways[name] = ocena
        
    return pathways

class SetSig(object):

    def __init__(self, **kwargs):
        for a,b in kwargs.items():
            setattr(self, a, b)

    def __call__(self, example):
        return setSig_example(self.learndata, example, self.genesets)

class SetSigLearner(object):

    def __call__(self, data, matcher, geneSets, minSize=3, maxSize=1000, minPart=0.1, classValues=None):
        data, oknames, gsetsnum = selectGenesetsData(data, matcher, geneSets, \
            minSize=minSize, maxSize=maxSize, minPart=minPart, classValues=classValues)
        return SetSig(learndata=data, genesets=gsetsnum)

if __name__ == "__main__":

    data = Orange.data.Table("iris")
    gsets = obiGeneSets.collections({
        #"ALL": ['sepal length', 'sepal width', 'petal length', 'petal width'],
        "f3": ['sepal length', 'sepal width', 'petal length'],
        "l3": ['sepal width', 'petal length', 'petal width'],
        })

    fp = 120
    ldata = orange.ExampleTable(data.domain, data[:fp])
    tdata = orange.ExampleTable(data.domain, data[fp:])

    matcher = obiGene.matcher([])

    choosen_cv = ["Iris-setosa", "Iris-versicolor"]
    #ass = AssessLearner()(data, matcher, gsets, rankingf=AT_loessLearner())
    ass = AssessLearner()(data, matcher, gsets, minPart=0.0)
    #ass = MeanLearner()(data, matcher, gsets, default=False)
    #ass = CORGsLearner()(data, matcher, gsets)
    #ass = MedianLearner()(data, matcher, gsets)
    #ass = PLSLearner()(data, matcher, gsets, classValues=choosen_cv, minPart=0.0)
    #ass = SetSigLearner()(ldata, matcher, gsets, classValues=choosen_cv, minPart=0.0)
    #ass = PCALearner()(ldata, matcher, gsets, classValues=choosen_cv, minPart=0.0)
    #ass = GSALearner()(ldata, matcher, gsets, classValues=choosen_cv, minPart=0.0)

    ar = defaultdict(list)
    for d in (list(ldata) + list(tdata))[:5]:
        for a,b in ass(d).items():
            ar[a].append(b)

    def pp1(ar):
        ol =  sorted(ar.items())
        print '\n'.join([ a.id + ": " +str(b) for a,b in ol])

    pp1(ar)




## Automatically adapted for numpy.oldnumeric Oct 04, 2007 by 

"""
<name>ANOVA</name>
<description>Single Sample T-test, One/Two Way Analysis of Variance.</description>
<icon>icons/ChipANOVA.png</icon>
<priority>1070</priority>
<contact>Peter Juvan (peter.juvan@fri.uni-lj.si)</contact>
<prototype>1</prototype>
"""

from __future__ import absolute_import

import numpy.oldnumeric as Numeric, numpy.oldnumeric.ma as MA
import scipy.stats

from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets.OWWidget import *

from .. import Anova
from .OWDataFiles import DataFiles, ExampleSelection

class OWANOVA(OWWidget):
    settingsList  = ["anovaType", "compareToValue", "_interaction", "selectorA", "selectorB", "selectorI", "alphaA", "alphaB", "alphaI", "autoUpdateSelName", "sendNotSelectedData", "sendProbabilities", "commitOnChange"]

    def __init__(self, parent=None, signalManager = None):
        OWWidget.__init__(self, parent, signalManager, 'ANOVA')
        # input / output data: [("name1", [orange.ExampleTable1a,...]), ("name2", [orange.ExampleTable2a,...])]
        self.inputs = [("Structured Data", DataFiles, self.onDataInput)]
        self.outputs = [("Example Selection", ExampleSelection, Default), ("Selected Structured Data", DataFiles, Default), ("Other Structured Data", DataFiles)]

        # data, p-values, selected examples
        self.dataStructure = None   # input data
        self.numExamples = 0
        self.numVariables = 0
        self.ps = None              # p-values: 2D Numeric.array of shape (3, numExamples)
        self.selectorName = ""      # for Example Selection output: (self.selectorName, [0,1,0,...])

        # Settings
        self.anovaType = 0      # 0: single-sample t-test, 1: one-way (A), 2: one-way (B), 3: two-way (A,B), 4: ful factorial (A, B, A*B)
        self.compareToValue = 0 # single sample t-test, value to compare to
        self._interaction = 0    # 0: no interaction, 1: test for interaction effect (set this value manually !!!)
        self.selectorA = True
        self.selectorB = False
        self.selectorI = False
        self.alphaA = "0.05"
        self.alphaB = "0.05"
        self.alphaI = "0.05"
        self.autoUpdateSelName = 1
        self.sendNotSelectedData = 1
        self.sendProbabilities = 0
        self.commitOnChange = 0
        self.loadSettings()

        # GUI
        self.mainArea.setFixedWidth(0)
        ca = self.controlArea
       
        # info
        box = OWGUI.widgetBox(ca, "Info")
        #gl.addWidget(box,0,0)
        self.infoa = OWGUI.label(box, self, 'No data on input.')
        self.infob = OWGUI.label(box, self, "")
        self.infoc = OWGUI.label(box, self, "")

        # ANOVA type
        # group selection
        anovaTypes = ["Single sample t-test", 
            "Single-factor (A, variables)", 
            "Single-factor (B, data sets)", 
            "Two-factor", 
            "Two-factor with interaction effect"]

        self.boxAnovaType = OWGUI.widgetBox(ca, "Anova Type")
        self.anovaTypeS = OWGUI.radioButtonsInBox(self.boxAnovaType, self, "anovaType", btnLabels=anovaTypes)
        
        self.boxAnovaType.setDisabled(1)

        self.boxCompareTo = OWGUI.widgetBox(self.boxAnovaType)
        OWGUI.lineEdit(self.boxCompareTo, self, "compareToValue", callback=self.onCompareToChange, label="compare to")

        # selection of examples
        self.boxSelection = OWGUI.widgetBox(ca, "Example Selection")
        self.lblNumGenes = []   # list of labels

        # selector A
        self.boxSelectorA = OWGUI.widgetBox(self.boxSelection)
        self.cbSelectorA = OWGUI.checkBox(self.boxSelectorA, self, "selectorA", "Factor A (variables)", callback=self.onSelectionChange,
                                          tooltip='H0: The mean does not depend on factor A (represented by variables).')

        frmA = OWGUI.widgetBox(self.boxSelectorA)
        leA = OWGUI.lineEdit(frmA, self, "alphaA", orientation="horizontal", callback=lambda x=0: self.onAlphaChange(x), label= "p <= ")
        self.lblNumGenes.append(OWGUI.label(frmA, self, ""))

        # selector B
        self.boxSelectorB = OWGUI.widgetBox(self.boxSelection)
        self.cbSelectorB = OWGUI.checkBox(self.boxSelectorB, self, "selectorB", "Factor B (data sets)", callback=self.onSelectionChange,
                                          tooltip='H0: The mean does not depend on factor B (represented by data sets).')
 
        frmB = OWGUI.widgetBox(self.boxSelectorB)
        leB = OWGUI.lineEdit(frmB, self, "alphaB", orientation="horizontal", callback=lambda x=1: self.onAlphaChange(x), label= "p <= ")
        self.lblNumGenes.append(OWGUI.label(frmB, self, ""))

        # selector I
        self.boxSelectorI = OWGUI.widgetBox(self.boxSelection)
        self.cbSelectorI = OWGUI.checkBox(self.boxSelectorI, self, "selectorI", "Interaction (variables * data sets)", callback=self.onSelectionChange,
                                          tooltip='H0: There is no interaction between factor A and factor B.')
 
        frmI = OWGUI.widgetBox(self.boxSelectorI)
        leI = OWGUI.lineEdit(frmI, self, "alphaI", orientation="horizontal", callback=lambda x=2: self.onAlphaChange(x), label= "p <= ")
        self.lblNumGenes.append(OWGUI.label(frmI, self, ""))

        # output
        box = OWGUI.widgetBox(ca, "Output")
        self.leSelectorName = OWGUI.lineEdit(box, self, 'selectorName', label='Selector Name: ')
        self.leSelectorName.setReadOnly(self.autoUpdateSelName)
        OWGUI.checkBox(box, self, 'autoUpdateSelName', 'Automatically update selector name', callback=self.onAutoUpdateSelNameChange)
        OWGUI.checkBox(box, self, 'sendNotSelectedData', 'Send not selected data', callback=self.onSendNotSelectedChange)
        OWGUI.checkBox(box, self, 'sendProbabilities', 'Show p-values', callback=self.onSendProbabilitiesChange)
        OWGUI.checkBox(box, self, 'commitOnChange', 'Commit data on selection change', callback=lambda: self.onCommit(self.commitOnChange))
        self.btnCommit = OWGUI.button(box, self, "Commit", callback=self.onCommit)

        # enable/disable anova type box, example selection box, commit button, update the number of examples for individual selectors
        self.updateAnovaTypeBox()
        self.updateSelectorBox()
        self.updateSelectorInfos()
        self.updateSelectorName()

        self.resize(283, self.sizeHint().height())


    def onDataInput(self, structuredData):
        """handles input data; sets self.dataStructure, self.numExamples, self.numVariables and self.ps;
        updates info, calls updateAnovaTypeBox(), runs ANOVA and sends out new data.
        """
        self.dataStructure = structuredData
        self.numExamples = 0
        self.numVariables = 0
        self.ps = None
        if structuredData:
            numFiles = reduce(lambda a,b: a+len(b[1]), structuredData, 0)
            lenSD = len(structuredData)
            self.infoa.setText("%d set%s, total of %d data file%s." % (lenSD, ["","s"][lenSD!=1], numFiles, ["","s"][numFiles!=1]))
            numExamplesList = []
            numVariablesList = []
            # construct a list of ExampleTable lengths and a list of number of variables
            for (name, etList) in structuredData:
                for et in etList:
                    numExamplesList.append(len(et))
                    numVariablesList.append(len(et.domain.variables))
            # test that all ExampleTables consist of equal number of examples and variables
            if len(numExamplesList) == 0 or Numeric.add.reduce(Numeric.equal(numExamplesList, numExamplesList[0])) != len(numExamplesList):
                self.dataStructure = None
                self.numExamples = -1
                self.infob.setText("Error: data files contain unequal number of examples, aborting ANOVA computation.")
                self.infoc.setText('')
            elif len(numVariablesList) == 0 or Numeric.add.reduce(Numeric.equal(numVariablesList, numVariablesList[0])) != len(numVariablesList):
                self.dataStructure = None
                self.numVariables = -1
                self.infob.setText("Error: data files contain unequal number of variables, aborting ANOVA computation.")
                self.infoc.setText('')
            else:
                self.numExamples = numExamplesList[0]
                self.numVariables = numVariablesList[0]
                self.infob.setText("%d variable%s, %d example%s in each file." % (self.numVariables, ["","s"][self.numVariables!=1], self.numExamples, ["","s"][self.numExamples!=1]))
                if self.numExamples > 0:
                    self.infoc.setText('Press Commit button to start ANOVA computation.')
                else:
                    self.infoc.setText('')
                self.boxAnovaType.setEnabled(1)
                self.boxSelection.setEnabled(1)
                self.btnCommit.setEnabled(True)
        else:
            self.infoa.setText('No data on input.')
            self.infob.setText('')
            self.infoc.setText('')
        # enable/disable anova type selection depending on the type of input data
        self.updateAnovaTypeBox()
        self.updateSelectorBox()
        if self.autoUpdateSelName:
            self.updateSelectorName()
        # run ANOVA
        if self.commitOnChange:
            self.runANOVA()
            self.senddata()
        self.updateSelectorInfos()

        
    def runANOVA(self):
        """converts structured data [(name, [orngET1, orngET2, ...]),...] to a 3D masked array
        with the following axes: 0: examples, 1: variables, 2: ExampleTables;
        runs ANOVA computations and sets self.ps;
        """
        if self.dataStructure and self.numExamples > 0:
            ma3d = MA.zeros((self.numExamples, self.numVariables, reduce(lambda a,b: a+len(b[1]), self.dataStructure, 0)), Numeric.Float) * MA.masked
            groupLens = []
            etIdx = 0
            for dsName, etList in self.dataStructure:
                for et in etList:
                    ma3d[:,:,etIdx] = et.toNumpyMA("ac")[0]
                    etIdx += 1
                groupLens.append(len(etList))

            #print "ma3d SHAPE", ma3d.shape
            #print "ma3d from top", ma3d[0,:,:]
            # run ANOVA
            self.infoc.setText('ANOVA computation started...')
            self.progressBarInit()
            pbStep = 100./self.numExamples
            self.ps = Numeric.ones((3, self.numExamples), Numeric.Float)
            if self.anovaType >= 3:
                ps = self.anova2(ma3d, groupLens, self.anovaType==4, repMeasuresOnA=False, callback=lambda: self.progressBarAdvance(pbStep))
                for rIdx in range(ps.shape[0]):
                    self.ps[rIdx] = ps[rIdx]
            elif self.anovaType == 2:
                self.ps[1] = self.anova1B(ma3d, groupLens, repMeasures=False, callback=lambda: self.progressBarAdvance(pbStep))
            elif self.anovaType == 1:
                self.ps[0] = self.anova1A(ma3d, repMeasures=False, callback=lambda: self.progressBarAdvance(pbStep))
            elif self.anovaType == 0:
                try:
                    compToVal = float(self.compareToValue)
                except:
                    print "Warning: cannot convert %s to float, using 0" % str(self.compareToValue)
                    self.compareToValue = 0
                    compToVal = 0
                self.ps[0] = self.ttest_ssmpl(ma3d, compToVal, callback=lambda: self.progressBarAdvance(pbStep))
            self.progressBarFinished()


    def ttest_ssmpl(self, ma3d, compToVal, callback):
        """conducts single-sample t-test on individual examples wrt factor A (variables, ma3d axis 1);
        returns Numeric array of p-values in shape (1, numExamples).
        """
        ps = -1*Numeric.ones((ma3d.shape[0],), Numeric.Float)
        for eIdx in range(ma3d.shape[0]):
            data = Numeric.asarray(MA.transpose(ma3d[eIdx]).compressed())
            if len(data) >= 2:
                try:
                    ps[eIdx] = scipy.stats.ttest_1samp(data, compToVal)[1]
                except:
                    print "Warning: zero variance, check the example %i:" % eIdx, data
                    ps[eIdx] = 1.0
            else:
                ps[eIdx] = 1.0
            callback()
        return ps

    def anova1A(self, ma3d, repMeasures, callback):
        """conducts one-way ANOVA on individual examples wrt factor A (variables, ma3d axis 1);
        returns Numeric array of p-values in shape (1, numExamples).
        """
        ps = -1*Numeric.ones((ma3d.shape[0],), Numeric.Float)
        if repMeasures:
            fAnova = Anova.AnovaRM12LR
        else:
            fAnova = Anova.Anova1wayLR_2D
        for eIdx in range(ma3d.shape[0]):
            an = fAnova(MA.transpose(ma3d[eIdx]))
            ps[eIdx] = an.Fprob
            callback()
        return ps

    def anova1B(self, ma3d, groupLens, repMeasures, callback):
        """conducts one-way ANOVA on individual examples wrt factor B (data sets);
        ma3d axis 2 also contains replicas according to groupLens;
        returns Numeric array of p-values in shape (1, numExamples).
        WARNING: works slower than anova1A because it requires to copy 1D array to 2D array
                 although we could use Anova1wayLR instead of Anova1wayLR_2D, but not for repeated measures
                 additionaly, Anova1wayLR_2D handles missing factor levels correctly, which is not the case for Anova1wayLR
        """
        ps = -1*Numeric.ones((ma3d.shape[0],), Numeric.Float)
        # groupLens [2,3,4] -> groupInd [[0,1],[2,3,4],[5,6,7,8]]
        if repMeasures:
            fAnova = Anova.AnovaRM12LR
        else:
            fAnova = Anova.Anova1wayLR_2D
        grpLensAcc = Numeric.concatenate([[0],Numeric.add.accumulate(groupLens)])
        grpInd = map(lambda i,j: range(i, j), grpLensAcc[:-1], grpLensAcc[1:])
        for eIdx in range(ma3d.shape[0]):
            m2 = MA.zeros((max(groupLens)*ma3d.shape[1], len(groupLens)), Numeric.Float) * MA.masked # axis0: replicas, axis1: factor B levels
            for groupIdx,takeInd in enumerate(grpInd):
                m2[:groupLens[groupIdx]*ma3d.shape[1], groupIdx] = MA.ravel(ma3d[eIdx].take(takeInd, 1))
            an = fAnova(m2)
            ps[eIdx] = an.Fprob
            callback()
        return ps

    def anova2(self, ma3d, groupLens, addInteraction, repMeasuresOnA, callback):
        """Conducts two-way ANOVA on individual examples;
        returns a Numeric array of p-values in shape (2, numExamples) or (3, numExamples), depending whether we test for interaction;
        Note: levels of factors A and B that cause empty cells are removed prior to conducting ANOVA.
        """
        groupLens = Numeric.asarray(groupLens)
        # arrays to store p-vals
        if addInteraction:
            ps = Numeric.ones((3, ma3d.shape[0]), Numeric.Float)
        else:
            ps = Numeric.ones((2, ma3d.shape[0]), Numeric.Float)
        # decide between non-repeated / repeated measures ANOVA for factor time
        if repMeasuresOnA:
            fAnova = Anova.AnovaRM12LR
        else:
            fAnova = Anova.Anova2wayLR
        # check for empty cells for all genes at once and remove them
        tInd2rem = []
        ax2Ind = Numeric.concatenate(([0], Numeric.add.accumulate(groupLens)))
        for aIdx in range(ma3d.shape[1]):
            for rIdx in range(groupLens.shape[0]):
                if Numeric.add.reduce(MA.count(ma3d[:,aIdx,ax2Ind[rIdx]:ax2Ind[rIdx+1]],1)) == 0:
                    tInd2rem.append(aIdx)
                    break
        if len(tInd2rem) > 0:
            print "Warning: removing time indices %s for all genes" % (str(tInd2rem))
            tInd2keep = range(ma3d.shape[1])
            for aIdx in tInd2rem:
                tInd2keep.remove(aIdx)
            ma3d = ma3d.take(tInd2keep, 1)
        # for each gene...
        for eIdx in range(ma3d.shape[0]):
            # faster check for empty cells for that gene -> remove time indices with empty cells
            ma2d = ma3d[eIdx]
            cellCount = MA.zeros((ma2d.shape[0], groupLens.shape[0]), Numeric.Int)
            for g,(i0,i1) in enumerate(zip(ax2Ind[:-1], ax2Ind[1:])):
                cellCount[:,g] = MA.count(ma2d[:,i0:i1], 1)
            ma2dTakeInd = Numeric.logical_not(Numeric.add.reduce(Numeric.equal(cellCount,0),1)) # 1 where to take, 0 where not to take
            if Numeric.add.reduce(ma2dTakeInd) != ma2dTakeInd.shape[0]:
                print "Warning: removing time indices %s for gene %i" % (str(Numeric.compress(ma2dTakeInd == 0, Numeric.arange(ma2dTakeInd.shape[0]))), eIdx)
                ma2d = MA.compress(ma2dTakeInd, ma2d, 0)
            an = fAnova(ma2d, groupLens, addInteraction, allowReductA=True, allowReductB=True)
            ps[:,eIdx] = an.ps
            callback()
        return ps


    def updateAnovaTypeBox(self):
        """enables/disables: - anova type selection box;
                             - example selection box;
                             - selectors A, B and I
                             - Commit button
        """
        if self.dataStructure and self.numExamples > 0:
            # enable anova type box and commit button
            self.boxAnovaType.setEnabled(1)
            self.btnCommit.setEnabled(1)
            # select appropriate anova type
            if len(self.dataStructure) == 1 and self.numVariables == 1:
                # single-sample t-test (factor A)
                self.anovaType = 0
            elif len(self.dataStructure) == 1 and self.numVariables > 1:
                # single-factor (A) ANOVA
                self.anovaType = 1
            elif len(self.dataStructure) > 1 and self.numVariables == 1:
                # single-factor (B) ANOVA
                self.anovaType = 2
            elif len(self.dataStructure) > 1 and self.numVariables > 1:
                # two-factor ANOVA
                self.anovaType = int(self._interaction) + 3
            # enable/disable appropriate anova type radio buttons
            if self.anovaType <= 2:
                for i in range(5):
                    self.boxAnovaType.buttons[i].setEnabled(self.anovaType == i)
            else:
                self.boxAnovaType.buttons[0].setEnabled(0)
                for i in range(1,5):
                    self.boxAnovaType.buttons[i].setEnabled(1)
            # enable/disable compareTo lineEdit
            self.boxCompareTo.setEnabled(self.anovaType == 0)
        else:
            # disable anova type box and commit button
            self.boxAnovaType.setDisabled(1)
            self.btnCommit.setDisabled(1)


    def updateSelectorBox(self):
        """enables / disables individual selectors
        """
        if self.dataStructure and self.numExamples > 0:
            # enable example selection box
            self.boxSelection.setEnabled(1)
            # enable/disable selectors A, B and I
            self.boxSelectorA.setEnabled(self.anovaType != 2)
            self.boxSelectorB.setEnabled(self.anovaType >= 2)
            self.boxSelectorI.setEnabled(self.anovaType == 4)
        else:
            # disable example selection box
            self.boxSelection.setDisabled(1)


    def updateSelectorInfos(self, selectorIdx=None):
        """updates the number of examples that match individual selectors;
        if selectorIdx is given, updates only the corresponding info.
        """
        if not selectorIdx:
            selectorInd = range(3)
        else:
            selectorInd = [selectorIdx]
        alphas = [self.alphaA, self.alphaB, self.alphaI]
        for si in selectorInd:
            try:
                alpha = float(alphas[si])
                ps = self.ps[si]
            except:
                alpha = None
                ps = None
            if ps != None and alpha != None and self.anovaType in [[0,1,3,4],[2,3,4],[4]][si]:
                numSelected = Numeric.add.reduce(Numeric.less_equal(self.ps[si], alpha))
                self.lblNumGenes[si].setText('  (%d example%s)' % (numSelected, ['', 's'][numSelected!=1]))
            else:
                self.lblNumGenes[si].setText('  (no examples)')


    def senddata(self):
        """computes selectionList, partitions the examples and updates infoc;
        sends out selectionList and selected/other dataStructure or None;
        """
##        if self.dataStructure and self.ps:
        if self.dataStructure and self.ps.shape[1]:
            # set selectionList
            alphas = [self.alphaA, self.alphaB, self.alphaI]
            selectors = [self.selectorA, self.selectorB, self.selectorI]
            selectionList = Numeric.ones((self.numExamples,))
            for si in range(3):
                try:
                    if selectors[si] and self.anovaType in [[0,1,3,4],[2,3,4],[4]][si]:
                        selectionList = Numeric.logical_and(selectionList, Numeric.less_equal(self.ps[si], float(alphas[si])))
                except:
                    pass
            self.infoc.setText('Sending out data...')
            
            if self.sendProbabilities:
                # create example table with probabilities
                print self.ps
                print Numeric.transpose(self.ps).shape
                etProb = orange.ExampleTable(orange.Domain([orange.FloatVariable("Factor A p-val"),orange.FloatVariable("Factor B p-val"),orange.FloatVariable("Interaction p-val")]), Numeric.transpose(self.ps))
                # in etProb, convert p-val to meta attribute
                domProb = orange.Domain([])
                domProb.addmetas(dict(zip([orange.newmetaid(),orange.newmetaid(),orange.newmetaid()], etProb.domain.variables)))
                etProb = orange.ExampleTable(domProb, etProb)
            else:
                # create new etProb without attributes/metas and of length equal to etProb
                etProb = orange.ExampleTable(orange.Domain([]), Numeric.zeros((selectionList.shape[0],0)))

            # partition dataStructure and send out data
            selectionList = selectionList.tolist()
            self.send("Example Selection", (self.selectorName, selectionList))
            dataStructS = []
            dataStructN = []
            self.progressBarInit()

            if self.sendNotSelectedData:
                pbStep = 50./len(self.dataStructure)
            else:
                pbStep = 100./len(self.dataStructure)

            for (dsName, etList) in self.dataStructure:
                etListS = [et.select(selectionList) for et in etList]
                for i in range(len(etList)):
                    # append probabilities (if etProb not empty)
                    etListS[i] = orange.ExampleTable([etListS[i], etProb.select(selectionList)])
                    # add name
                    etListS[i].name = etList[i].name
                dataStructS.append((dsName, etListS))
                self.progressBarAdvance(pbStep)
            self.send("Selected Structured Data", dataStructS)

            if self.sendNotSelectedData:
                for (dsName, etList) in self.dataStructure:
                    etListN = [et.select(selectionList, negate=1) for et in etList]
                    for i in range(len(etList)):
                        # append probabilities (if etProb not empty)
                        etListN[i] = orange.ExampleTable([etListN[i], etProb.select(selectionList, negate=1)])
                        # add name
                        etListN[i].name = etList[i].name
                    dataStructN.append((dsName, etListN))
                    self.progressBarAdvance(pbStep)
                self.send("Other Structured Data", dataStructN)
            else:
                self.send("Other Structured Data", None)

            self.progressBarFinished()
            # report the number of selected examples
            numExamples = Numeric.add.reduce(Numeric.greater(selectionList, 0))
            self.infoc.setText('Total of %d example%s match criteria.' % (numExamples, ['', 's'][numExamples!=1]))
        else:
            self.send("Example Selection", None)
            self.send("Selected Structured Data", None)
            self.send("Other Structured Data", None)
            

    def updateSelectorName(self):
        """update selector name shown in selector edit box
        """
        if self.dataStructure:
            if self.anovaType == 0:
                s = '1 smpl. t-test, compared to %s' % self.compareToValue
            else:
                s = 'ANOVA'
            s += " (%s)" % reduce(lambda a,b: a + ", " + b[0], self.dataStructure, "")[2:]
            if self.selectorA and self.anovaType in [0,1,3,4]:
                s += ", pA<%s" % self.alphaA
            if self.selectorB and self.anovaType in [2,3,4]:
                s += ", pB<%s" % self.alphaB
            if self.selectorI and self.anovaType == 4:
                s += ", pI<%s" % self.alphaI
            self.selectorName = s.strip()
        else:
            self.selectorName = ""


    #==========================================================================
    # Event handlers
    #==========================================================================

    def onCompareToChange(self):
        """handles changes of ANOVA type:
            - resets self.ps;
            - updates infoc
            - calls updateSelectorInfos()
        runs ANOVA and sends out new data;
        """
        if self.anovaType == 0:
            self.ps = None
            if self.autoUpdateSelName:
                self.updateSelectorName()
            if self.commitOnChange:
                self.runANOVA()
                self.senddata()
            elif self.dataStructure and self.numExamples > 0:
                self.infoc.setText('Press Commit button to start ANOVA computation.')
            self.updateSelectorInfos()


    def onAnovaType(self):
        """handles changes of ANOVA type:
            - resets self.ps;
            - calls updateSelectorBox()
            - updates infoc
            - calls updateSelectorInfos()
        runs ANOVA and sends out new data;
        """
        #print "self.anovaType", self.anovaType
        self._interaction = self.anovaType == 4
        self.ps = None
        self.updateSelectorBox()
        if self.autoUpdateSelName:
            self.updateSelectorName()
        if self.commitOnChange:
            self.runANOVA()
            self.senddata()
        elif self.dataStructure and self.numExamples > 0:
            self.infoc.setText('Press Commit button to start ANOVA computation.')
        self.updateSelectorInfos()


    def onInteraction(self):
        """handles clicks on interaction checkbox:
            - resets self.ps;
            - enables/disables selector I,
            - updates infoc
        runs ANOVA and sends out new data
        """
        self.ps = None
        self.boxSelectorI.setEnabled(self.interaction)
        if self.autoUpdateSelName:
            self.updateSelectorName()
        if self.commitOnChange:
            self.runANOVA()
            self.senddata()
        elif self.dataStructure and self.numExamples > 0:
            self.infoc.setText('Press Commit button to start ANOVA computation.')
        self.updateSelectorInfos()


    def onSelectionChange(self):
        """handles changes in example selector checkboxes;
        sends out new data;
        """
        if self.autoUpdateSelName:
            self.updateSelectorName()
        if self.commitOnChange:
            self.senddata()


    def onAlphaChange(self, selectorIdx):
        """handles changes in example selector alphas;
        prints number of selected examples for individual selectors and sends out new data;
        """
        if self.autoUpdateSelName:
            self.updateSelectorName()
        if self.commitOnChange:
            self.senddata()
        self.updateSelectorInfos(selectorIdx)


    def onAutoUpdateSelNameChange(self):
        """handles clicks on auto update selector name checkbox
        """
        self.leSelectorName.setReadOnly(self.autoUpdateSelName)

            
    def onSendNotSelectedChange(self):
        """handles clicks on sendNotSelectedData checkbox
        """
        if self.commitOnChange:
            self.senddata()

    def onSendProbabilitiesChange(self):            
        """handles clicks on show p-values checkbox
        """
        if self.commitOnChange:
            self.senddata()
            
    def onCommit(self, commit=True):
        """handles Commit clicks; runs ANOVA (if not already computed) and sends out data;
        """
        if commit:
            if self.dataStructure:
                if self.ps == None:
                    self.runANOVA()
                self.senddata()
            self.updateSelectorInfos()


if __name__=="__main__":
    from . import OWDataFiles, OWDataFilesSelector
    from Orange.orng import orngSignalManager
    from Orange.OrangeWidgets.Data import OWDataTable

    signalManager = orngSignalManager.SignalManager(0)
    a=QApplication(sys.argv)
    an=OWANOVA(signalManager = signalManager)

    an.show()
    
    df = OWDataFiles.OWDataFiles(signalManager = signalManager)
    df.loadData("/home/marko/anova/smallchipdata")
    df.show()

    signalManager.addWidget(an)
    signalManager.addWidget(df)
    signalManager.setFreeze(1)
    signalManager.addLink(df, an, 'Structured Data', 'Structured Data', 1)

    # data files selector, data table
    dfs = OWDataFilesSelector.OWDataFilesSelector(signalManager = signalManager)
    signalManager.addWidget(dfs)
    dfs.show()
    signalManager.addLink(an, dfs, 'Selected Structured Data', 'Structured Data', 1)
    dt = OWDataTable.OWDataTable(signalManager = signalManager)
    signalManager.addWidget(dt)
    signalManager.addLink(dfs, dt, 'Examples', 'Examples', 1)
    signalManager.setFreeze(0)
    dt.show()

    a.exec_()
    an.saveSettings()



"""
Getting genesets from KEGG and GO.

Maintainer: Marko Toplak
"""

from __future__ import absolute_import, with_statement

import cPickle as pickle, os, tempfile, sys
from collections import defaultdict

import orange
from Orange.orng import orngServerFiles

from . import obiGO, obiKEGG, obiTaxonomy

sfdomain = "gene_sets"

def nth(l,n):
    return [ a[n] for a in l]

class GeneSet(object):

    def __init__(self, genes=None, name=None, id=None, \
        description=None, link=None, organism=None, hierarchy=None, pair=None):
        """
        pair can be (id, listofgenes) - it is used before anything else.
        """
        if genes == None:
            genes = []

        self.hierarchy = hierarchy
        self.genes = set(genes)
        self.name = name
        self.id = id
        self.description = description
        self.link = link
        self.organism = organism

        if pair:
            self.id, self.genes = pair[0], set(pair[1])

    """
    the following functions are needed for sets of gene sets to be able
    to assess equality
    """

    def __hash__(self):
        return self.id.__hash__() + self.name.__hash__()

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__
        else:
            return False

    def __ne__(self, other):
        return not self.__eq__(other)

    def size(self):
        return len(self.genes)

    def cname(self, source=True, name=True):
        """ Constructs a gene set name with the hierarchy. """
        oname = self.id
        if source and self.hierarchy:
            oname = "[ " + ", ".join(self.hierarchy) + " ] " + oname
        if name and self.name:
            oname = oname + " " + self.name
        return oname

    def to_odict(self, source=True, name=True):
        """
        Returns a pair (id, listofgenes), like in old format.
        """
        return self.cname(source=source, name=name), self.genes

    def __repr__(self):
        return "GeneSet(" + ", ".join( [ 
            "id=" + str(self.id),
            "genes=" + str(self.genes),
            "name=" + str(self.name),
            "link=" + str(self.link),
            "hierarchy=" + str(self.hierarchy)
        ]) + ")"

class GeneSetIDException(Exception):
    pass

class GeneSets(set):
    
    def __init__(self, input=None):
        """
        odict are genesets in old dict format.
        gs are genesets in new format
        """
        if input != None and len(input) > 0:
            self.update(input)

    def update(self, input):
        from . import obiGeneSets
        if isinstance(input, obiGeneSets.GeneSets):
            super(GeneSets, self).update(input)
        elif hasattr(input, "items"):
            for i, g in input.items():
                self.add(obiGeneSets.GeneSet(pair=(i, g)))
        else:
            for i in input:
                if isinstance(i, obiGeneSets.GeneSet):
                    self.add(i)
                else:
                    i, g = i
                    self.add(obiGeneSets.GeneSet(pair=(i, g)))

    def to_odict(self):
        """ Return gene sets in old dictionary format. """
        return dict(gs.to_odict() for gs in self)

    def set_hierarchy(self, hierarchy):
        """ Sets hierarchy for all gene sets """
        for gs in self:
            gs.hierarchy = hierarchy

    def __repr__(self):
        return "GeneSets(" + set.__repr__(self) + ")"

    def common_org(self):
        """ Returns the common organism. """
        if len(self) == 0:
            raise GenesetRegException("Empty gene sets.")

        organisms = set(a.organism for a in self)

        try:
            return only_option(organisms)
        except:
            raise GenesetRegException("multiple organisms: " + str(organisms))

    def hierarchies(self):
        """ Returns all hierachies """
        if len(self) == 0:
            raise GenesetRegException("Empty gene sets.")
        return set(a.hierarchy for a in self)

    def common_hierarchy(self):
        hierarchies = self.hierarchies()

        def common_hierarchy1(hierarchies):
            def hier(l): return set(map(lambda x: x[:currentl], hierarchies))
            currentl = max(map(len, hierarchies))
            while len(hier(currentl)) > 1:
                currentl -= 1
            return only_option(hier(currentl))

        return common_hierarchy1(hierarchies)

    def split_by_hierarchy(self):
        """ Splits gene sets by hierarchies. """
        from . import obiGeneSets
        hd = dict((h,obiGeneSets.GeneSets()) for h in  self.hierarchies())
        for gs in self:
            hd[gs.hierarchy].add(gs)
        return hd.values()

def goGeneSets(org):
    """Returns gene sets from GO."""
    from . import obiGeneSets

    ontology = obiGO.Ontology()
    annotations = obiGO.Annotations(org, ontology=ontology)

    genesets = []
    link_fmt = "http://amigo.geneontology.org/cgi-bin/amigo/term-details.cgi?term=%s"
    for termn, term in ontology.terms.items():
        genes = annotations.GetAllGenes(termn)
        hier = ("GO", term.namespace)
        if len(genes) > 0:
            gs = obiGeneSets.GeneSet(id=termn, name=term.name, genes=genes, hierarchy=hier, organism=org, link=link_fmt % termn) 
            genesets.append(gs)

    return obiGeneSets.GeneSets(genesets)

def keggGeneSets(org):
    """
    Returns gene sets from KEGG pathways.
    """
    from . import obiKEGG2 as obiKEGG, obiGeneSets
    
    kegg = obiKEGG.KEGGOrganism(org)

    genesets = []
    for id in kegg.pathways():
        pway = obiKEGG.KEGGPathway(id)
        hier = ("KEGG","pathways")
        gs = obiGeneSets.GeneSet(id=id,
                                 name=pway.title,
                                 genes=kegg.get_genes_by_pathway(id),
                                 hierarchy=hier,
                                 organism=org,
                                 link=pway.link)
        genesets.append(gs)

    return obiGeneSets.GeneSets(genesets)

def omimGeneSets():
    """
    Return gene sets from OMIM (Online Mendelian Inheritance in Man) diseses
    """
    from . import obiOMIM
    genesets = [GeneSet(id=disease.id, name=disease.name, genes=obiOMIM.disease_genes(disease), hierarchy=("OMIM",), organism="9606",
                    link=("http://www.ncbi.nlm.nih.gov/entrez/dispomim.cgi?id=" % disease.id if disease.id else None)) \
                    for disease in obiOMIM.diseases()]
    return GeneSets(genesets)

def miRNAGeneSets(org):
    """
    Return gene sets from miRNA targets
    """
    from . import obimiRNA, obiKEGG
    org_code = obiKEGG.from_taxid(org)
    link_fmt = "http://www.mirbase.org/cgi-bin/mirna_entry.pl?acc=%s"
    mirnas = [(id, obimiRNA.get_info(id)) for id in obimiRNA.ids(org_code)]
    genesets = [GeneSet(id=mirna.matACC, name=mirna.matID, genes=mirna.targets.split(","), hierarchy=("miRNA", "Targets"),
                        organism=org, link=link_fmt % mirna.matID) for id, mirna in mirnas]
    return GeneSets(genesets)

def go_miRNASets(org, ontology=None, enrichment=True, pval=0.05, treshold=0.04):
    from . import obimiRNA, obiGO
    mirnas = obimiRNA.ids(int(org))
    if ontology is None:
        ontology = obiGO.Ontology()
         
    annotations = obiGO.Annotations(org, ontology=ontology)
    
    go_sets = obimiRNA.get_GO(mirnas, annotations, enrichment=enrichment, pval=pval, goSwitch=False)
    print go_sets
    
    go_sets = obimiRNA.filter_GO(go_sets, annotations, treshold=treshold)
    
    from . import obiGeneSets as gs
    link_fmt = "http://amigo.geneontology.org/cgi-bin/amigo/term-details.cgi?term=%s"
    gsets = [gs.GeneSet(id=key, name=ontology[key].name, genes=value, hierarchy=("miRNA", "go_sets",),
                        organism=org, link=link_fmt % key) for key, value in go_sets.items()]
    gset = gs.GeneSets(gsets)
    return gset


def loadGMT(contents, name):
    """
    Eech line consists of tab separated elements. First is
    the geneset name, next is it's description.
    
    For now the description is skipped.
    """

    from . import obiGeneSets

    def hline(s):
        tabs = [tab.strip() for tab in s.split("\t")]
        return obiGeneSets.GeneSet(id=tabs[0], description=tabs[1],
                                   hierarchy=(name,), genes=tabs[2:])

    def handleNELines(s, fn):
        """
        Run function on nonempty lines of a string.
        Return a list of results for each line.
        """
        lines = (l.strip() for l in s.splitlines())
        return [fn(l) for l in lines if l]

    return obiGeneSets.GeneSets(handleNELines(contents, hline))

"""
We have multiple paths for gene set data:
buffer/bigfiles/gene_sets
and
buffer/gene_sets_local
both have available.txt
"""

def omakedirs(dir):
    try:
        os.makedirs(dir)
    except OSError:
        pass

def local_path():
    """ Returns local path for gene sets. Creates it if it does not exists
    yet. """
    from Orange.orng import orngEnviron
    pth = os.path.join(orngEnviron.directoryNames["bufferDir"], "gene_sets_local")
    omakedirs(pth)
    return pth

def build_index(dir):
    """ Returns gene set availability index for some folder. """
    pass

class GenesetRegException(Exception): pass

def only_option(a):
    if len(a) == 1:
        return list(a)[0]
    else:
        raise Exception()

def filename(hierarchy, organism):
    """ Obtain a filename for given hierarchy and organism. """
    return "gs_" + "_._".join(hierarchy + \
        (organism if organism != None else "",)) + ".pck"

def filename_parse(fn):
    """ Returns a hierarchy and the organism from the filename."""
    fn = fn[3:-4]
    parts = fn.split("_._")
    hierarchy = tuple(parts[:-1])
    org = parts[-1] if parts[-1] != "" else None
    return hierarchy, org

def is_genesets_file(fn):
    return fn.startswith("gs_") and fn.endswith(".pck")

def list_local():
    """ Returns available gene sets from the local repository:
    a list of (hierarchy, organism, on_local) """
    pth = local_path()
    gs_files = filter(is_genesets_file, os.listdir(pth))
    return [ filename_parse(fn) + (True,) for fn in gs_files ]
    
def list_serverfiles_from_flist(flist):
    gs_files = filter(is_genesets_file, flist)
    localfiles = set(orngServerFiles.listfiles(sfdomain))
    return [ filename_parse(fn) + \
        ((True,) if fn in localfiles else (False,)) for fn in gs_files ]

def list_serverfiles_conn(serverfiles=None):
    """ Returns available gene sets from the server files
    repository: a list of (hierarchy, organism, on_local) """
    if serverfiles == None:
        serverfiles = orngServerFiles.ServerFiles()
    flist = serverfiles.listfiles(sfdomain)
    return list_serverfiles_from_flist(flist)

def list_serverfiles():
    fname = orngServerFiles.localpath_download(sfdomain, "index.pck")
    flist = pickle.load(open(fname, 'r'))
    return list_serverfiles_from_flist(flist)

def list_all():
    """
    return a list of (hier, org, avalable_locally)
    If something for a specific (hier, org) is not downloaded
    yet, show it as not-local. """
    flist = list_local() + list_serverfiles()
    d = {}
    for h,o,local in flist:
        d[h,o] = min(local, d.get((h,o),True))
    return [ (h,o,local) for (h,o),local in d.items() ]

def update_server_list(serverfiles_upload, serverfiles_list=None):
    if serverfiles_list == None:
        serverfiles_list = orngServerFiles.ServerFiles()

    flist = map(lambda x: filename(*x[:2]), list_serverfiles_conn(serverfiles_list))

    tfname = pickle_temp(flist)
    
    try:
        fn = "index.pck"
        title = "Gene sets: index"
        tags = [ "gene sets", "index", "essential" ]
        serverfiles_upload.upload(sfdomain, fn, tfname, title, tags)
        serverfiles_upload.unprotect(sfdomain, fn)
    except Exception,e:
        raise e
    finally:
        os.remove(tfname)

def register_local(genesets):
    """ Registers using the common hierarchy and organism. """
    pth = local_path()

    org = genesets.common_org()
    hierarchy = genesets.common_hierarchy()
    fn = filename(hierarchy, org)

    with open(os.path.join(pth, fn), "w") as f:
        pickle.dump(genesets, f)

    return fn

def pickle_temp(obj):
    """ Pickle a file to a temporary file returns its name """
    fd,tfname = tempfile.mkstemp()
    os.close(fd)
    f = open(tfname, 'wb')
    pickle.dump(obj, f)
    f.close()
    return tfname

def register_serverfiles(genesets, serverFiles):
    """ Registers using the common hierarchy and organism. """
    org = genesets.common_org()
    hierarchy = genesets.common_hierarchy()
    fn = filename(hierarchy, org)

    #save to temporary file
    tfname = pickle_temp(genesets)
    
    try:
        taxname = obiTaxonomy.name(org)
        title = "Gene sets: " + ", ".join(hierarchy) + \
            ((" (" + taxname + ")") if org != None else "")
        tags = list(hierarchy) + [ "gene sets", taxname ] + \
            ([ "essential" ] if org in obiTaxonomy.essential_taxids() else [] )
        serverFiles.upload(sfdomain, fn, tfname, title, tags)
        serverFiles.unprotect(sfdomain, fn)
    finally:
        os.remove(tfname)

    update_server_list(serverFiles)

def register(genesets, serverFiles=None):
    """
    Hierarchy is induced from the gene set names.
    """
    if serverFiles == None:
        register_local(genesets)
    else:
        register_serverfiles(genesets, serverFiles)

def build_hierarchy_dict(files):
    hierd = defaultdict(list)
    for ind,f in enumerate(files):
        hier, org = f
        for i in range(len(hier)+1):
            hierd[(hier[:i], org)].append(ind)
    return hierd

def load_local(hierarchy, organism):
    files = map(lambda x: x[:2], list_local())

    hierd = build_hierarchy_dict(files)

    out = GeneSets()
    for (h, o) in [ files[i] for i in hierd[(hierarchy, organism)]]:
        fname = os.path.join(local_path(), filename(h, o))
        out.update(pickle.load(open(fname, 'r')))
    return out

def load_serverfiles(hierarchy, organism):
    files = map(lambda x: x[:2], list_serverfiles())
    hierd = build_hierarchy_dict(files)

    out = GeneSets()
    for (h, o) in [ files[i] for i in hierd[(hierarchy, organism)]]:
        fname = orngServerFiles.localpath_download(sfdomain, 
            filename(h, o))
        out.update(pickle.load(open(fname, 'r')))
    return out

def load(hierarchy, organism):
    """ First try to load from the local registred folder, then
    from the server files. """
    ret = load_local(hierarchy, organism)
    ret.update(load_serverfiles(hierarchy, organism))
    return ret

def collections(*args):
    """
    Input is a list of collections.
    Collection can either be a tuple (hierarchy, orgranism), where
    hierarchy is a tuple also.
    """
    from . import obiGeneSets
    result = obiGeneSets.GeneSets()

    for collection in args:
        try:
            result.update(collection)
        except ValueError:
            if issequencens(collection): #have a hierarchy, organism specification
                new = load(*collection)
                result.update(new)
            else:
                if collection.lower()[-4:] == ".gmt": #format from webpage
                    result.update(loadGMT(open(collection,"rt").read(), collection))
                else:
                    raise Exception("collection() accepts files in .gmt format only.")

    return result

def issequencens(x):
    "Is x a sequence and not string ? We say it is if it has a __getitem__ method and it is not an instance of basestring."
    return hasattr(x, '__getitem__') and not isinstance(x, basestring)

class TException(Exception): pass

def upload_genesets(rsf):
    """
    Builds the default gene sets and 
    """
    orngServerFiles.update_local_files()

    genesetsfn = [ keggGeneSets, goGeneSets, miRNAGeneSets]
    organisms = obiTaxonomy.common_taxids()
    for fn in genesetsfn:
        for org in organisms:
        #for org in [ "9606" ]:
            print "Uploading ORG", org, fn
            try:
                genesets = fn(org).split_by_hierarchy()
                for gs in genesets:
                    print "registering", gs.common_hierarchy()
                    register_serverfiles(gs, rsf)
                    print "successful", gs.common_hierarchy()
            except Exception, e:
                print "Not successful"

if __name__ == "__main__":

    #gs = keggGeneSets("9606")
    akegg = collections((("KEGG",),"9606"))
    a = collections({"PATH": ["g1","g2","g3"]}, "steroltalk.gmt", akegg)
    for e in a:
        print e
    #print len(collections(keggGeneSets("9606"),(("KEGG",),"9606"), "C5.BP.gmt"))
    #print len(collections((("KEGG",),"9606"), "C5.BP.gmt"))
    #print sorted(list_all())
    #print len(collections((("KEGG",),"9606"), (("GO",), "9606"), "C5.BP.gmt"))
    #register_local(keggGeneSets("9606"))
    #register_local(goGeneSets("9606"))
    #register_serverfiles(gs, rsf)
    #print list_serverfiles_conn()
    #print "Server list from index", list_serverfiles()
    rsf = orngServerFiles.ServerFiles(username=sys.argv[1], password=sys.argv[2])
    upload_genesets(rsf)



## Automatically adapted for numpy.oldnumeric Oct 04, 2007 by 

import numpy.oldnumeric as Numeric, numpy.oldnumeric.ma as MA
import statc
    

def kNNimputeMA(arr2d, K=20, callback=None):
    """Returns a new 2D MA.array with missing values imputed from K nearest neighbours.
    Find K rows (axis 0) with the most similar values where similarity measure corresponds to weighted Euclidean distance.
    Imputed value = weighted average of the corresponding values of K nearest neighbours,
    where weights equal to tricubic distribution of distances to all rows.
    Impute missing rows by average over all rows.
    Version: 30.8.2005
    """
    arr2d = MA.asarray(arr2d)
    assert len(arr2d.shape) == 2, "2D array expected"
    # make a copy for imputation
    aImp2 = MA.array(arr2d)
    # leave out columns with 0 known values (columnInd: non-zero columns)
    columnCond = Numeric.greater(MA.count(arr2d, axis=0), 0)
    columnIndAll = Numeric.arange(arr2d.shape[1])
    columnInd = Numeric.compress(columnCond, columnIndAll)
    # impute the rows where 0 < #known_values < #non_zero_columns, i.e. exclude the rows with 0 and all (non-zero-column) values
    countByRows = MA.count(arr2d, axis=1)
    for rowIdx in Numeric.compress(Numeric.logical_and(Numeric.greater(countByRows, 0), Numeric.less(countByRows, columnInd.shape[0])), Numeric.arange(arr2d.shape[0])):
        rowResized = MA.resize(arr2d[rowIdx], arr2d.shape)
        diff = arr2d - rowResized
        distances = MA.sqrt(MA.add.reduce((diff)**2, 1) / MA.count(diff, axis=1))
        # nearest neighbours row indices (without the current row index)
        indSorted = MA.argsort(distances)[1:]
        distSorted = distances.take(indSorted)
        # number of distances different from MA.masked
        numNonMasked = distSorted.shape[0] - Numeric.add.reduce(Numeric.asarray(MA.getmaskarray(distSorted), Numeric.Int))
        # number of distances to account for (K or less)
        if numNonMasked > 1:
            weightsSorted = MA.power(1-MA.power(distSorted/distSorted[numNonMasked-1],3),3) # tricubic distribution of all weights
        else:
            weightsSorted = Numeric.ones(distSorted.shape[0])
        # compute average for each column separately in order to account for K non-masked values
        colInd4CurrRow = Numeric.compress(Numeric.logical_and(MA.getmaskarray(arr2d[rowIdx]), columnCond), columnIndAll)
        for colIdx in colInd4CurrRow:
            # column values sorted by distances
            columnVals = arr2d[:,colIdx].take(indSorted)
            # take only those weights where columnVals does not equal MA.masked
            weightsSortedCompressed = MA.compress(1-MA.getmaskarray(columnVals), weightsSorted)
            # impute from K (or possibly less) values
            aImp2[rowIdx,colIdx] = MA.average(columnVals.compressed()[:K], weights=weightsSortedCompressed[:K])
        if callback:
            callback()
    # impute the unknown rows with average profile
    avrgRow = MA.average(arr2d, 0)
    for rowIdx in Numeric.compress(Numeric.equal(countByRows, 0), Numeric.arange(arr2d.shape[0])):
        aImp2[rowIdx] = avrgRow
        if callback:
            callback()
    return aImp2


def loessMA(m, windowSize, axis=0, approxMasked=True, verbose=False, callback=None):
    """Returns a new array with values at the given axis smoothed by loess;
    if approxMasked==True: the masked values are approximated by loess;
    assumes equidistant spacing of points on the given axis.
    """
    assert 0 < windowSize <= m.shape[axis]+0.1, "0 < windowSize[%s] <= 1 OR windowSize in range(1.1,m.shape[axis]+1) expected, got %f" % ("%", windowSize)
    m = MA.asarray(m)
    if m.dtype.char <> Numeric.Float:
        m = m.astype(Numeric.Float)
    shp_other = list(m.shape)
    shp_other.pop(axis)
    # get a transposed and reshaped mask and data from m; if m.mask() == None, construct a new array of zeros
    mask = Numeric.reshape(Numeric.transpose(MA.getmaskarray(m), [axis] + range(0,axis) + range(axis+1,len(m.shape))), (m.shape[axis], Numeric.multiply.reduce(shp_other)))
    data = MA.reshape(MA.transpose(m, [axis] + range(0,axis) + range(axis+1,len(m.shape))), (m.shape[axis], Numeric.multiply.reduce(shp_other)))
    maskInv = -1*(mask-1)
    xall = Numeric.arange(data.shape[0])
    xallList = xall.tolist()
    for ii in Numeric.compress(Numeric.add.reduce(maskInv,0) > 1, range(data.shape[1])):    # run loess if the profile contains more than 2 values
        try:
            data[:,ii] = MA.array(statc.loess(zip(MA.compress(maskInv[:,ii], xall).tolist(), MA.compress(maskInv[:,ii], data[:,ii]).tolist()), xallList, windowSize))[:,1]
        except:
            if verbose:
                print "Warning: loessMA: could not loess axis %i index %i" % (axis, ii)
        if callback:
            callback()
    if not approxMasked:
        data = MA.array(data, mask=mask)
    return MA.transpose(MA.reshape(data, [m.shape[axis]] + shp_other), [axis] + range(0,axis) + range(axis+1,len(m.shape)))



if __name__=="__main__":
    import numpy.oldnumeric.random_array as RandomArray, time

#   10x4 array    
##    m2 = MA.asarray(RandomArray.random((10, 4)))
##    m2[0,0] = MA.masked
##    m2[1,1] = MA.masked
##    m2[2,3] = MA.masked
##    m2[3,:2] = MA.masked
##    m2[4,2:] = MA.masked
##    m2[5,:] = MA.masked
##    m2[5,1] = 0.42423
##    m2[6,:] = MA.masked    
##    m2i3 = kNNimputeMA(m2, 3)
##    m2i30 = kNNimputeMA(m2, 30)

    # 0 column
##    m3 = MA.concatenate([m2, MA.zeros((10,1))*MA.masked], 1)
##    m3i3 = kNNimputeMA(m3, 3)

    # 3 rows only
##    m34 = MA.asarray(RandomArray.random((3,4)))
##    m34[0,:2] = MA.masked
##    m34[1,1:3] = MA.masked
##    m34[-1] = MA.masked
##    m34i = kNNimputeMA(m34,3)

    # large array
##    shp = (1000,10)
##    l1 = RandomArray.random(shp)
##    m1 = RandomArray.randint(0,2,shp)
##    r1 = MA.array(l1, mask=m1)
##    print time.ctime()
##    r1i = kNNimputeMA(r1)
##    print time.ctime()




import string
import re
import sys
import os.path

def usage():
   print "%s geo-file-name orange-file-name" % os.path.basename(sys.argv[0])

if len(sys.argv)<>3:
   usage()
   sys.exit(0)
   
inname = sys.argv[1]
outname = sys.argv[2]

#outname = "out.tab"
#inname = "GDS1962.soft.tab"

attvalue = re.compile("(.*) = (.*)")

def geo_process_file(inname, choice=None):
   print "Processing: %s" % inname
   f = open(inname)
   subsets = {}
   data = {}
   genes = []
   while True:
      line = f.readline()
      if "!dataset_table_begin" in line:
         # which sample to include
         samples = []
         for v in subsets.values():
            samples.extend(v)
         for s in samples:
            data[s] = []

         # classification class(sample)
         classification = {}
         for c, ss in subsets.items():
            for s in ss:
               classification[s] = c

         # header with sample names          
         line = f.readline() 
         line = line[:-1]
         header = string.split(line, "\t")
         accept = dict([(h,i) for i,h in enumerate(header[2:]) if h in samples])

         # read the data
         while True:
            line = f.readline() # read data line
            if "!dataset_table_end" in line:
               f.close()
               return data, genes, classification
            line = line[:-1]
            items = string.split(line, "\t")
            genes.append(items[1]) # 1=gene name, 0=spot id
            items = items[2:] # only the data values
            for k,i in accept.items():
               data[k].append(items[i])

         
      if "!subset_description" in line:
         m = attvalue.search(line)
         sname = m.group(2)
         line = f.readline()
         m = attvalue.search(line)
         if not choice or sname in choice:
            subsets[sname] = string.split(m.group(2), ",")
            
      if "!dataset_table_end" in line:
         break
      
   return subsets

#rawdata, geneids = geo_process_file(inname, ['tumor grade IV', 'tumor grade II', 'tumor grade III'])
#rawdata, geneids, classification = geo_process_file("GDS330.soft.txt")
rawdata, geneids, classification = geo_process_file(inname)


genepos = {}
for i, g in enumerate(geneids):
   genepos[g] = genepos.get(g, []) + [i]

fout = open(outname, "w")
print "Saving to:  %s" % outname
fout.write("id\toutcome\t" + "\t".join(geneids) + "\n")
fout.write("string\td" + "\tc"*len(geneids) + "\n")
fout.write("meta\tclass" + "\t"*len(geneids) + "\n")
for s, items in rawdata.items():
   fout.write("%s\t%s\t" % (s, classification[s]) + "\t".join(items) + "\n")
fout.close()




from __future__ import absolute_import

import numpy

import orange, statc

from . import stats

def mean(l):
    return float(sum(l))/len(l)

class MA_pearsonCorrelation:
    """
    Calling an object of this class computes Pearson correlation of all
    attributes against class.
    """
    def __call__(self, i, data):
        dom2 = orange.Domain([data.domain.attributes[i]], data.domain.classVar)
        data2 = orange.ExampleTable(dom2, data)
        a,c = data2.toNumpy("A/C")
        return numpy.corrcoef(c,a[:,0])[0,1]

class MA_signalToNoise:
    """
    Returns signal to noise measurement: difference of means of two classes
    divided by the sum of standard deviations for both classes. 

    Usege similar to MeasureAttribute*.

    Standard deviation used for now returns minmally 0.2*|mi|, where mi=0 is adjusted to mi=1
    (as in gsea implementation).

    Can work only on data with two classes. If there are multiple class, then
    relevant class values can be specified on object initialization.
    By default the relevant classes are first and second class value
    from the domain.
    """

    def __init__(self, a=None, b=None):
        """
        a and b are choosen class values.
        """
        self.a = a
        self.b = b

    def __call__(self, i, data):
        cv = data.domain.classVar
        #print data.domain

        if self.a == None: self.a = cv.values[0]
        if self.b == None: self.b = cv.values[1]

        def stdev(l):
            return statc.std(l)

        def mean(l):
            return statc.mean(l)

        def stdevm(l):
            m = mean(l)
            std = stdev(l)
            #return minmally 0.2*|mi|, where mi=0 is adjusted to mi=1
            return max(std, 0.2*abs(1.0 if m == 0 else m))

        def avWCVal(value):
            return [ex[i].value for ex in data if ex[-1].value == value and not ex[i].isSpecial() ]

        exa = avWCVal(self.a)
        exb = avWCVal(self.b)

        try:
            rval = (mean(exa)-mean(exb))/(stdevm(exa)+stdevm(exb))
            return rval
        except:
            #return some "middle" value -
            #TODO rather throw exception? 
            return 0

class MA_t_test(object):
    def __init__(self, a=None, b=None, prob=False):
        self.a = a
        self.b = b
        self.prob = prob
    def __call__(self, i, data):
        cv = data.domain.classVar
        #print data.domain

        #for faster computation. to save dragging many attributes along
        dom2 = orange.Domain([data.domain[i]], data.domain.classVar)
        data = orange.ExampleTable(dom2, data)
        i = 0

        if self.a == None: self.a = cv.values[0]
        if self.b == None: self.b = cv.values[1]

        def avWCVal(value):
            return [ex[i].value for ex in data if ex[cv] == value and not ex[i].isSpecial() ]

        exa = avWCVal(self.a)
        exb = avWCVal(self.b)

        try:
            t, prob = stats.lttest_ind(exa, exb)
            return prob if self.prob else t
        except:
            return 1.0 if self.prob else 0.0

class MA_fold_change(object):
    def __init__(self, a=None, b=None):
        self.a = a
        self.b = b
    def __call__(self, i, data):
        cv = data.domain.classVar
        #print data.domain

        #for faster computation. to save dragging many attributes along
        dom2 = orange.Domain([data.domain[i]], data.domain.classVar)
        data = orange.ExampleTable(dom2, data)
        i = 0

        if self.a == None: self.a = cv.values[0]
        if self.b == None: self.b = cv.values[1]

        def avWCVal(value):
            return [ex[i].value for ex in data if ex[cv] == value and not ex[i].isSpecial() ]

        exa = avWCVal(self.a)
        exb = avWCVal(self.b)

        try:
            return mean(exa)/mean(exb)
        except:
            return 1

class MA_anova(object):
    def __init__(self, prob=False):
        self.prob = prob
    def __call__(self, i, data):
        cv = data.domain.classVar
        #print data.domain

        #for faster computation. to save dragging many attributes along
        dom2 = orange.Domain([data.domain[i]], data.domain.classVar)
        data = orange.ExampleTable(dom2, data)
        i = 0

        def avWCVal(value):
            return [ex[i].value for ex in data if ex[cv] == value and not ex[i].isSpecial() ]

        data = [avWCVal(val) for val in cv.values]

        try:
            f, prob = stats.lF_oneway(*tuple(data))
            return prob if self.prob else f
        except:
            return 1.0 if self.prob else 0.0

import numpy as np
import numpy.ma as ma

class ExpressionSignificance_Test(object):
    def __new__(cls, data, useAttributeLabels, **kwargs):
        self = object.__new__(cls)
        if kwargs:
            self.__init__(data, useAttributeLabels)
            return self.__call__(**kwargs)
        else:
            return self
    
    def __init__(self, data, useAttributeLabels=False):
        self.data = data
        self.useAttributeLabels = useAttributeLabels
        self.attr_labels, self.data_classes = self._data_info(data)
        self.attributes = [attr for attr in self.data.domain.attributes if attr.varType in [orange.VarTypes.Continuous, orange.VarTypes.Discrete]]
        self.classes = np.array(self.attr_labels if useAttributeLabels else self.data_classes)
        self.keys = range(len(data)) if useAttributeLabels else self.attributes
        self.array, _, _ = data.toNumpyMA()
        if self.useAttributeLabels:
            self.array = ma.transpose(self.array)
#        self.dim = 1 if useAttributeLabels else 0  
        self.dim = 0
        
    def _data_info(self, data):
        return [set(attr.attributes.items()) for attr in data.domain.attributes], [ex.getclass() for ex in data] if data.domain.classVar else [None]*len(data)
        
    def test_indices(self, target, classes=None):
        classes = self.classes if classes is None else classes
        
        def target_set(target):
            if isinstance(target, tuple):
                return set([target])
            else:
                assert(isinstance(target, set))
                return target
            
        if self.useAttributeLabels:
            if isinstance(target, list):
                ind = [[i for i, cl in enumerate(self.classes) if target_set(t).intersection(cl)] for t in target]
            else:
                target = target_set(target)
                
                ind1 = [i for i, cl in enumerate(self.classes) if target.intersection(cl)]
                ind2 = [i for i, cl in enumerate(self.classes) if not target.intersection(cl)]
                ind = [ind1, ind2]
        else:
            if isinstance(target, list):
                ind = [ma.nonzero(self.classes == t)[0] for t in target]
            else:
                if isinstance(target, (basestring, orange.Variable)):
                    target = set([target])
                else:
                    assert(isinstance(target, set))
                target = list(target)
                ind1 = [i for i, cl in enumerate(self.classes) if cl in target]
                ind2 = [i for i, cl in enumerate(self.classes) if cl not in target]
                ind = [ind1, ind2]
                
        return ind
    
    def __call__(self, target):
        raise NotImplementedError()
    
    def null_distribution(self, num, *args, **kwargs):
        kwargs = dict(kwargs)
        advance = lambda: None
        if "advance" in kwargs:
            advance = kwargs["advance"]
            del kwargs["advance"]
        results = []
        originalClasses = self.classes.copy()
        for i in range(num):
            np.random.shuffle(self.classes)
            results.append(self.__call__(*args, **kwargs))
            advance()
        self.classes = originalClasses
        return results
    
class ExpressionSignificance_TTest(ExpressionSignificance_Test):
    def __call__(self, target):
        ind1, ind2 = self.test_indices(target)
        t, pval = attest_ind(self.array[ind1, :], self.array[ind2, :], dim=self.dim)
        return zip(self.keys,  zip(t, pval))
        
class ExpressionSignificance_FoldChange(ExpressionSignificance_Test):
    def __call__(self, target):
        ind1, ind2 = self.test_indices(target)
        a1, a2 = self.array[ind1, :], self.array[ind2, :]
        fold = ma.mean(a1, self.dim)/ma.mean(a2, self.dim)
        return zip(self.keys, fold)
    
class ExpressionSignificance_SignalToNoise(ExpressionSignificance_Test):
    def __call__(self, target):
        ind1, ind2 = self.test_indices(target)
        a1, a2 = self.array[ind1, :], self.array[ind2, :]
        stn = (ma.mean(a1, self.dim) - ma.mean(a2, self.dim)) / (ma.sqrt(ma.var(a1, self.dim)) + ma.sqrt(ma.var(a2, self.dim)))
        return zip(self.keys, stn)
    
class ExpressionSignificance_ANOVA(ExpressionSignificance_Test):
    def __call__(self, target=None):
        if target is not None:
            indices = self.test_indices(target)
        else:
            indices = []
        f, prob = aF_oneway(*[self.array[ind, :] for ind in indices], **dict(dim=0))
        return zip(self.keys, zip(f, prob))
        
class ExpressionSignificance_ChiSquare(ExpressionSignificance_Test):
    def __call__(self, target):
        array = equi_n_discretization(self.array.copy(), intervals=5, dim=0)
        ind1, ind2 = self.test_indices(target)
        a1, a2 = array[ind1, :], array[ind2, :]
        dist1, dist2  = [], []
        dist = ma.zeros((array.shape[1], 2, 5))
        for i in range(5):
            dist1.append(ma.sum(ma.ones(a1.shape) * (a1 == i), 0))
            dist2.append(ma.sum(ma.ones(a2.shape) * (a2 == i), 0))
            dist[:, 0, i] = dist1[-1]
            dist[:, 1, i] = dist2[-1] 
        return zip(self.keys, achisquare_indtest(np.array(dist), dim=1))
        
class ExpressionSignificance_Info(ExpressionSignificance_Test):
    def __call__(self, target):
        array = equi_n_discretization(self.array.copy(), intervals=5, dim=1)
        
        ind1, ind2 = self.test_indices(target)
        a1, a2 = array[ind1, :], array[ind2, :]
        dist1, dist2 = [], []
        dist = ma.zeros((array.shape[1], 2, 5))
        for i in range(5):
            dist1.append(ma.sum(ma.ones(a1.shape) * (a1 == i), 0))
            dist2.append(ma.sum(ma.ones(a2.shape) * (a2 == i), 0))
            dist[:, 0, i] = dist1[-1]
            dist[:, 1, i] = dist2[-1]
        classinfo = entropy(np.array([len(ind1), len(ind2)]))
        E = ma.sum(entropy(dist, dim=1) * ma.sum(dist, 1), 1) / ma.sum(ma.sum(dist, 1), 1)
        return zip(self.keys, classinfo - E)
    
class ExpressionSignificance_MannWhitneyu(ExpressionSignificance_Test):
    def __call__(self, target):
        ind1, ind2 = self.test_indices(target)
        a, b = self.array[ind1, :], self.array[ind2, :]
#        results = [amannwhitneyu(a[:, i],b[:, i]) for i in range(a.shape[1])]
        results = [statc.mannwhitneyu(list(a[:, i]),list(b[:, i])) for i in range(a.shape[1])]
        
        return zip(self.keys, results)

def attest_ind(a, b, dim=None):
    """ Return the t-test statistics on arrays a and b over the dim axis.
    Returns both the t statistic as well as the p-value
    """
#    dim = a.ndim - 1 if dim is None else dim
    x1, x2 = ma.mean(a, dim), ma.mean(b, dim)
    v1, v2 = ma.var(a, dim), ma.var(b, dim)
    n1, n2 = (a.shape[dim], b.shape[dim]) if dim is not None else (a.size, b.size)
    df = float(n1+n2-2)
    svar = ((n1-1)*v1+(n2-1)*v2) / df
    t = (x1-x2)/ma.sqrt(svar*(1.0/n1 + 1.0/n2))
    if t.ndim == 0:
        return (t, statc.betai(0.5*df,0.5,df/(df+t**2)) if t is not ma.masked and df/(df+t**2) <= 1.0 else ma.masked)
    else:
        prob = [statc.betai(0.5*df,0.5,df/(df+tsq)) if tsq is not ma.masked and df/(df+tsq) <= 1.0 else ma.masked  for tsq in t*t]
        return t, prob

def aF_oneway(*args, **kwargs):
    dim = kwargs.get("dim", None)
    arrays = args
    means = [ma.mean(a, dim) for a in arrays]
    vars = [ma.var(a, dim) for a in arrays]
    lens = [ma.sum(ma.array(ma.ones(a.shape), mask=ma.asarray(a).mask), dim) for a in arrays]
    alldata = ma.concatenate(arrays, dim if dim is not None else 0)
    bign =  ma.sum(ma.array(ma.ones(alldata.shape), mask=alldata.mask), dim)
    sstot = ma.sum(alldata ** 2, dim) - (ma.sum(alldata, dim) ** 2) / bign
    ssbn = ma.sum([(ma.sum(a, dim) ** 2) / L for a, L in zip(arrays, lens)], dim)
#    print ma.sum(alldata, dim) ** 2 / bign, ssbn
    ssbn -= ma.sum(alldata, dim) ** 2 / bign
    sswn = sstot - ssbn
    dfbn = dfnum = float(len(args) - 1.0)
    dfwn = bign - len(args) # + 1.0
    F = (ssbn / dfbn) / (sswn / dfwn)
    if F.ndim == 0 and dfwn.ndim == 0:
        return (F,statc.betai(0.5 * dfwn, 0.5 * dfnum, dfwn/float(dfwn+dfnum*F)) if F is not ma.masked and dfwn/float(dfwn+dfnum*F) <= 1.0 \
                and dfwn/float(dfwn+dfnum*F) >= 0.0 else ma.masked)
    else:
        prob = [statc.betai(0.5 * dfden, 0.5 * dfnum, dfden/float(dfden+dfnum*f)) if f is not ma.masked and dfden/float(dfden+dfnum*f) <= 1.0 \
            and dfden/float(dfden+dfnum*f) >= 0.0 else ma.masked for dfden, f in zip (dfwn, F)]
        return F, prob
    
def achisquare_indtest(observed, dim=None):
    if observed.ndim == 2:
        observed = ma.array([observed])
        if dim is not None:
            dim += 1 
    if dim is None:
        dim = observed.ndim - 2
    rowtotal = ma.sum(observed, dim + 1)
    coltotal = ma.sum(observed, dim)
    total = ma.sum(rowtotal, dim)
    ones = ma.array(ma.ones(observed.shape))
    expected = ones * rowtotal.reshape(rowtotal.shape[:dim] + (-1, 1))
    a = ones * coltotal[..., np.zeros(observed.shape[dim], dtype=int),:]
    expected = expected * (a) / total.reshape((-1, 1, 1))
    chisq = ma.sum(ma.sum((observed - expected) ** 2 / expected, dim + 1), dim)
    return chisq
    
def equi_n_discretization(array, intervals=5, dim=1):
    count = ma.sum(ma.array(ma.ones(array.shape, dtype=int), mask=array.mask), dim)
    cut = ma.zeros(len(count), dtype=int)
    sarray = ma.sort(array, dim)
    r = count % intervals
    pointsshape = list(array.shape)
    pointsshape[dim] = 1
    points = []
    for i in range(intervals):
        cutend = cut + count / intervals + numpy.ones(len(r)) * (r > i)
        if dim == 1:
            p = sarray[range(len(cutend)), numpy.array(cutend, dtype=int) -1]
        else:
            p = sarray[numpy.array(cutend, dtype=int) -1, range(len(cutend))]
        points.append(p.reshape(pointsshape))
        cut = cutend
    darray = ma.array(ma.zeros(array.shape) - 1, mask=array.mask)
    darray[ma.nonzero(array <= points[0])] = 0
    for i in range(0, intervals):
        darray[ma.nonzero((array > points[i]))] = i + 1 
    return darray

def entropy(array, dim=None):
    if dim is None:
        array = array.ravel()
        dim = 0
    n = ma.sum(array, dim)
    array = ma.log(array) * array
    sum = ma.sum(array, dim)
    return (ma.log(n) - sum / n) / ma.log(2.0)

"""\
MA - Plot
=========

Functions for normalization of expression arrays and ploting
MA - Plots

Example::
    ## Load data from GEO
    >>> data = orange.ExampleTable("GDS1210.tab")
    ## Split data by columns into normal and cancer subsets
    >>> cancer, normal = data_split(data, [("disease state", "cancer"), ("disease state", "normal")])
    ## Convert to numpy MaskedArrays
    >>> cancer, normal = cancer.toNumpyMA("A")[0], normal.toNumpyMA("A")[0]
    ## Merge by averaging
    >>> cancer = merge_replicates(cancer)
    >>> normal = merge_replicates(normal)
    ## Plot MA-plot
    >>> MA_plot(cancer, normal)
    
"""

from Orange.orng import orngMisc
from numpy import median
def lowess(x, y, f=2./3., iter=3, progressCallback=None):
    """ Lowess taken from Bio.Statistics.lowess, modified to compute pairwise 
    distances inplace.
     
    lowess(x, y, f=2./3., iter=3) -> yest

    Lowess smoother: Robust locally weighted regression.
    The lowess function fits a nonparametric regression curve to a scatterplot.
    The arrays x and y contain an equal number of elements; each pair
    (x[i], y[i]) defines a data point in the scatterplot. The function returns
    the estimated (smooth) values of y.

    The smoothing span is given by f. A larger value for f will result in a
    smoother curve. The number of robustifying iterations is given by iter. The
    function will run faster with a smaller number of iterations.

    x and y should be numpy float arrays of equal length.  The return value is
    also a numpy float array of that length.

    e.g.
    >>> import numpy
    >>> x = numpy.array([4,  4,  7,  7,  8,  9, 10, 10, 10, 11, 11, 12, 12, 12,
    ...                 12, 13, 13, 13, 13, 14, 14, 14, 14, 15, 15, 15, 16, 16,
    ...                 17, 17, 17, 18, 18, 18, 18, 19, 19, 19, 20, 20, 20, 20,
    ...                 20, 22, 23, 24, 24, 24, 24, 25], numpy.float)
    >>> y = numpy.array([2, 10,  4, 22, 16, 10, 18, 26, 34, 17, 28, 14, 20, 24,
    ...                 28, 26, 34, 34, 46, 26, 36, 60, 80, 20, 26, 54, 32, 40,
    ...                 28, 26, 34, 34, 46, 26, 36, 60, 80, 20, 26, 54, 32, 40,
    ...                 32, 40, 50, 42, 56, 76, 84, 36, 46, 68, 32, 48, 52, 56,
    ...                 64, 66, 54, 70, 92, 93, 120, 85], numpy.float)
    >>> result = lowess(x, y)
    >>> len(result)
    50
    >>> print "[%0.2f, ..., %0.2f]" % (result[0], result[-1])
    [4.85, ..., 84.98]
    """
    n = len(x)
    r = min(int(numpy.ceil(f*n)), n - 1)
    
#    h = [numpy.sort(numpy.abs(x-x[i]))[r] for i in range(n)]
#    h, xtmp = numpy.zeros_like(x), numpy.zeros_like(x)
#    for i in range(n):
#        xtmp = numpy.abs(x - x[i], xtmp)
#        h[i] = numpy.sort(xtmp)[r]
#    w = numpy.clip(numpy.abs(([x]-numpy.transpose([x]))/h),0.0,1.0)
    dist = [x] - numpy.transpose([x])
    dist = numpy.abs(dist, dist)
    dist.sort(axis=1)
    h = dist[:, r]
    del dist

    w = [x]-numpy.transpose([x])
    w /= h
    w = numpy.abs(w, w)
    w = numpy.clip(w, 0.0, 1.0, w)
#    w = 1-w*w*w
    w **= 3
    w *= -1
    w += 1
#    w = w*w*w
    w **= 3
    yest = numpy.zeros(n)
    delta = numpy.ones(n)
    milestones = orngMisc.progressBarMilestones(iter*n)
    for iteration in range(iter):
        for i in xrange(n):
            weights = delta * w[:,i]
            weights_mul_x = weights * x
            b1 = numpy.ma.dot(weights,y)
            b2 = numpy.ma.dot(weights_mul_x,y)
            A11 = sum(weights)
            A12 = sum(weights_mul_x)
            A21 = A12
            A22 = numpy.ma.dot(weights_mul_x,x)
            determinant = A11*A22 - A12*A21
            beta1 = (A22*b1-A12*b2) / determinant
            beta2 = (A11*b2-A21*b1) / determinant
            yest[i] = beta1 + beta2*x[i]
            if progressCallback and (iteration*n + i) in milestones:
                progressCallback((100. * iteration*n + i) /  (iter * n))
        residuals = y-yest
        s = median(abs(residuals))
        delta[:] = numpy.clip(residuals/(6*s),-1,1)
        delta[:] = 1-delta*delta
        delta[:] = delta*delta
    return yest



def lowess2(x, y, xest, f=2./3., iter=3, progressCallback=None):
    """Returns estimated values of y in data points xest (or None if estimation fails).
    Lowess smoother: Robust locally weighted regression.
    The lowess function fits a nonparametric regression curve to a scatterplot.
    The arrays x and y contain an equal number of elements; each pair
    (x[i], y[i]) defines a data point in the scatterplot. The function returns
    the estimated (smooth) values of y.

    The smoothing span is given by f. A larger value for f will result in a
    smoother curve. The number of robustifying iterations is given by iter. The
    function will run faster with a smaller number of iterations.
    
    Taken from Peter Juvan's numpyExtn.py, modified for numpy, computes pairwise
    distances inplace
    """
    x = numpy.asarray(x, 'f')
    y = numpy.asarray(y, 'f')
    xest = numpy.asarray(xest, 'f')
    n = len(x)
    nest = len(xest)
    r = min(int(numpy.ceil(f*n)),n-1) # radius: num. of points to take into LR
#    h = [numpy.sort(numpy.abs(x-x[i]))[r] for i in range(n)]    # distance of the r-th point from x[i]
    dist = [x] - numpy.transpose([x])
    dist = numpy.abs(dist, dist)
    dist.sort(axis=1)
    h = dist[:, r]
    del dist # to free memory
    w = [x] - numpy.transpose([x])
    w /= h
    w = numpy.abs(w, w)
    w = numpy.clip(w, 0.0, 1.0, w)
#    w = numpy.clip(numpy.abs(([x]-numpy.transpose([x]))/h),0.0,1.0)
    w **= 3
    w *= -1
    w += 1
#    w = 1 - w**3 #1-w*w*w
    w **= 3
#    w = w**3 #w*w*w
#    hest = [numpy.sort(numpy.abs(x-xest[i]))[r] for i in range(nest)]    # r-th min. distance from xest[i] to x
    dist = [x] - numpy.transpose([xest])
    dist = numpy.abs(dist, dist)
    dist.sort(axis=1)
    hest = dist[:, r]
    del dist # to free memory
#    west = numpy.clip(numpy.abs(([xest]-numpy.transpose([x]))/hest),0.0,1.0)  # shape: (len(x), len(xest)
    west = [xest]-numpy.transpose([x])
    west /= hest
    west = numpy.abs(west, west)
    west = numpy.clip(west, 0.0, 1.0, west)
#    west = 1 - west**3 #1-west*west*west
    west **= 3
    west *= -1
    west += 1
#    west = west**3 #west*west*west
    west **= 3
    yest = numpy.zeros(n,'f')
    yest2 = numpy.zeros(nest,'f')
    delta = numpy.ones(n,'f')
    iter_count = iter*(nest + n) if iter > 1 else nest
    milestones = orngMisc.progressBarMilestones(iter_count)
    curr_iter = 0
    for iteration in range(iter):
        # fit xest
        for i in range(nest):
            weights = delta * west[:,i]
            b = numpy.array([numpy.sum(weights*y), numpy.sum(weights*y*x)])
            A = numpy.array([[numpy.sum(weights), numpy.sum(weights*x)], [numpy.sum(weights*x), numpy.sum(weights*x*x)]])
            beta = numpy.linalg.solve(A, b)
            yest2[i] = beta[0] + beta[1]*xest[i]
            if progressCallback and curr_iter in milestones:
                progressCallback(100. * curr_iter / iter_count)
            curr_iter += 1
                
        # fit x (to calculate residuals and delta)
        if iter > 1:
            for i in range(n):
                weights = delta * w[:,i]
                b = numpy.array([numpy.sum(weights*y), numpy.sum(weights*y*x)])
                A = numpy.array([[numpy.sum(weights), numpy.sum(weights*x)], [numpy.sum(weights*x), numpy.sum(weights*x*x)]])
                beta = numpy.linalg.solve(A,b)
                yest[i] = beta[0] + beta[1]*x[i]
                if progressCallback and curr_iter in milestones:
                    progressCallback(100. * curr_iter / iter_count)
                curr_iter += 1
            residuals = y-yest
            s = numpy.median(numpy.abs(residuals))
            delta = numpy.clip(residuals/(6*s), -1, 1)
            delta = 1-delta*delta
            delta = delta*delta
    return yest2


def attr_group_indices(data, label_groups):
    """ Return a two or more lists of indices into `data.domain` based on `label_groups`
    
    Example::
        cancer_indices, no_cancer_indices = attr_group_indices(data, [("disease state", "cancer"), ("disease state", "normal")])
    """
    ret = []
    for key, val in label_groups:
        ind = [i for i, attr in enumerate(data.domain.attributes) if attr.attributes.get(key, None) == val]
        ret.append(ind)
    return ret


def example_group_indices(data, attr, values):
    """ Return lists of indices into `data` for each `values` item that matches
    the example value at `attr` attribute
    
    Example::
        cls_ind1, cls_ind2 = example_group_indices(data, data.domain.classVar, ["Class 1", "Class 2"])
    """
    ret = [[] for _ in values]
    values_id = dict([(str(value), i) for i, value in enumerate(values)])
    for i, ex in enumerate(data):
        id = values_id.get(str(ex[attr]), None)
        if id is not None:
            ret[id].append(i)
    return ret
    
    
def data_group_split(data, label_groups):
    """ Split an `data` example table into two or more based on
    contents of iterable `label_groups` containing (key, value)
    pairs matching the labels of data attributes.
    
    Example::
        cancer, no_cancer = data_group_split(data, [("disease state", "cancer"), ("disease state", "normal")])
    """
    ret = []
    group_indices = attr_group_indices(data, label_groups)
    for indices in group_indices:
        attrs = [data.domain[i] for i in indices]
        domain = orange.Domain(attrs, data.domain.classVar)
        domain.addmetas(data.domain.getmetas())
        ret.append(orange.ExampleTable(domain, data))
    return ret
    
    
def select_indices(data, key, value, axis=1):
    """ Return indices into `data` (ExampleTable) along specified `axis`
    where:
        - if axis == 0 match data[i][key] == value
        - if axis == 1 match data.domain[i].attributes[key] == value 
    
    Example::
        cancer_ind = select_indices(data, key="disease state", value="cancer"), axis=1)
        normal_ind = select_indices(data, key="disease state", value=["normal"], axis=1) # value can be a list to specify more then one value
        
    """
    values = value if isinstance(value, list) else [value]
    if axis == 0:
        groups = example_group_indices(data, key, values)
    else:
        groups = attr_group_indices(data, [(key, val) for val in values])
        
    return sorted(reduce(set.union, groups, set()))


def select_data(data, key, value, axis=1):
    """ Return `data` (ExampleTable) subset along specified `axis` where
    where:
        - if axis == 0 match data[i][key] == value
        - if axis == 1 match data.domain[i].attributes[key] == value 
        .. note:: This preserves all meta attributes of the domain
    Example::
        cancer = select_data(data, "disease state", "cancer", axis=1)
        normal = select_data(data, "disease state", ["normal"], axis=1) # value can be a list to specify more then one value
        
    """
    indices = select_indices(data, key, value, axis)
    if axis == 0:
        examples = [data[i] for i in indices]
        return orange.ExampleTable(data.domain, examples)
    else:
        attrs = [data.domain[i] for i in indices]
        domain = orange.Domain(attrs, False)
        domain.addmetas(data.domain.getmetas())
        return orange.ExampleTable(domain, data)
    
    
def split_data(data, groups, axis=1):
    """ Split data (ExampleTable) along specified axis, where elements of 
    `groups` match `key` and `value` arguments of the `select_data`
    function 
    
    Example::
        cancer, normal = split_data(data, [("disease state", "cancer"), ("disease state", ["normal"])], axis=1)
    """
    res = []
    for key, value in groups:
        res.append(select_data(data, key, value, axis))
    return res

    
def geometric_mean(array):
    """ Return a geometric mean computed on a 1d masked array
    """
    array = numpy.ma.asanyarray(array)
    return numpy.power(reduce(lambda a,b: a*b, array.filled(1.), 1.0), 1./len(array))


def harmonic_mean(array):
    """ Return a harmonic mean computed ona a 1d masked array
    """
    array = numpy.ma.asanyarray(array)
    return len(array) / numpy.ma.sum(1. / array)


def merge_replicates(replicates, axis=0, merge_function=numpy.ma.average):
    """ Merge `replicates` (numpy.array) along `axis` using `merge_function`
    """
    return numpy.ma.apply_along_axis(merge_function, axis, replicates)


def ratio_intensity(G, R):
    """ return the log2(R/G), log10(R*G) as a tuple
    """
    log2Ratio = numpy.ma.log(R/G) / numpy.log(2)
    log10Intensity = numpy.ma.log10(R*G)
    return log2Ratio, log10Intensity


def MA_center_average(G, R):
    """ return the G, R by centering the average log2 ratio
    """
    center_est = numpy.ma.average(numpy.ma.log(R/G) / numpy.log(2))
    G = G * numpy.exp2(center_est)
    return G, R.copy()


def MA_center_lowess(G, R, f=2./3., iter=1, progressCallback=None):
    """ return the G, R by centering the average log2 ratio locally
    depending on the intensity using lowess (locally weighted linear regression)
    """
#    from Bio.Statistics.lowess import lowess
    ratio, intensity = ratio_intensity(G, R)
    valid = - (ratio.mask & intensity.mask)
    valid_ind = numpy.ma.where(valid)
    center_est = lowess(intensity[valid], ratio[valid], f=f, iter=iter, progressCallback=progressCallback)
    Gc, R = G.copy(), R.copy()
    Gc[valid] *= numpy.exp2(center_est)
    Gc.mask, R.mask = -valid, -valid
    return Gc, R


def MA_center_lowess_fast(G, R, f=2./3., iter=1, resolution=100, progressCallback=None):
    """return the G, R by centering the average log2 ratio locally
    depending on the intensity using lowess (locally weighted linear regression),
    approximated only in a limited resolution.
    """
    
    ratio, intensity = ratio_intensity(G, R)
    valid = - (ratio.mask & intensity.mask)
    resoluiton = min(resolution, len(intensity[valid]))
    hist, edges = numpy.histogram(intensity[valid], len(intensity[valid])/resolution)
    
    progressCallback2 = (lambda val: progressCallback(val/2)) if progressCallback else None 
    centered = lowess2(intensity[valid], ratio[valid], edges, f, iter, progressCallback=progressCallback2)

    progressCallback2 = (lambda val: progressCallback(50 + val/2)) if progressCallback else None
    centered = lowess2(edges, centered, intensity[valid], f, iter, progressCallback=progressCallback2)
    
    Gc, R = G.copy(), R.copy()
    Gc[valid] *= numpy.exp2(centered)
    Gc.mask, R.mask = -valid, -valid
    return Gc, R


def MA_plot(G, R, format="b."):
    """ Plot G, R on a MA-plot using matplotlib
    """
    import matplotlib.pyplot as plt
    ratio, intensity = ratio_intensity(G, R)
    plt.plot(intensity, ratio, format)
    plt.ylabel('M = log2(R/G')
    plt.xlabel('A = log10(R*G)')


def normalize_expression_data(data, groups, axis=1, merge_function=numpy.ma.average, center_function=MA_center_lowess_fast):
    """ A helper function that normalizes expression array example table, by centering the MA plot.
    
    """
    if isinstance(data, orange.ExampleTable):
        label_groups = [select_indices(data, key, value, axis) for key, value in groups]
        array, _, _ = data.toNumpyMA()
    
    merged = []
    for indices in label_groups:
        replicates = numpy.take(array, indices, axis=1)
        merged.append(merge_replicates(replicates, axis=1, merge_function=merge_function))
        
    ind1, ind2 = label_groups
    G, R = merged
    Gc, Rc = center_function(G, R)
    
    domain = orange.Domain(data.domain.attributes, data.domain.classVar)
    domain.addmetas(data.domain.getmetas())
    data = orange.ExampleTable(domain, data)
    
    GFactors = Gc/G
    
    if axis == 0:
        for i, gf in zip(ind1, GFactors):
            for attr in range(len(data[i])):
                if not data[i][attr].isSpecial():
                    data[i][attr] = float(data[i][attr]) * gf
    else:   
        for ex, gf in zip(data, GFactors):
            for i in ind1:
                if not ex[i].isSpecial():
                    ex[i] = float(ex[i]) * gf
    return data
    
    
def MA_zscore(G, R, window=1./5., padded=False, progressCallback=None):
    """ Return the Z-score of log2 fold ratio estimated from local
    distribution of log2 fold ratio values on the MA-plot
    """
    ratio, intensity = ratio_intensity(G, R)
    
    z_scores = numpy.ma.zeros(G.shape)
    sorted = list(numpy.ma.argsort(intensity))
    import math, random
    r = int(math.ceil(len(sorted)*window)) # number of window elements
    def local_indices(i, sorted):
        """ local indices in sorted (mirror padded if out of bounds)
        """
        start, end = i - r/2, i + r/2 + r%2
        pad_start , pad_end = [], []
        if start < 0:
            pad_start = sorted[:abs(start)]
            random.shuffle(pad_start)
            start = 0
        if end > len(sorted):
            pad_end = sorted[end - len(sorted):]
            random.shuffle(pad_end)
            end = len(sorted)
        
        if padded:
            return pad_start + sorted[start: end] + pad_end
        else:
            return sorted[start:end]
    
    milestones = orngMisc.progressBarMilestones(len(sorted))
    for i in range(len(sorted)):
        indices = local_indices(i, sorted)
        localRatio = numpy.take(ratio, indices)
        local_std = numpy.ma.std(localRatio)
        ind = sorted[i]
        z_scores[ind] = ratio[ind] / local_std
        if progressCallback and i in milestones:
            progressCallback(100. * i / len(sorted))
        
    z_scores._mask = - numpy.isfinite(z_scores)
    return z_scores
    
